#include <c4d.h>
#include "messiahC4Dscene_events_handler.h"
#include "messiahC4Dmhobject.h"
#include "messiahC4Dconstants.h"
#include "messiahSceneHook.h"

// old config file code, temporary
#include "ConfigData.h"

extern ConfigData *config;

// Copy Geometry to messiah (::geometryCopy)  /////////////////////////////////////////////////////////////////

bool messiahC4Dscene_events_handler::geometryCopy( mh::object &obj, int flags )
{
	char const * dbg_name = obj.name();

	mh::surface * surf		= obj.createSurface("tmp");	// we'll handle surfaces later, need at least one though
	// only copy geom over after object is loaded
	if( static_cast<mC4Dmhobject &>(obj).loading )
	{
		obj.createPoint( 0, 0, 0 );
		int pt = 0;
		obj.createPoly( 1, &pt, surf );
		obj.finalize( flags );
		return true;
	}

	BaseObject *	cache = static_cast<BaseObject *>(obj.d_data1);
	if( !cache ) return false;

	int i;

	// Create Points	/////////////////////////////////////////
	// scaling values as indicated by the user
	
	LONG		num_points	= ToPoint(cache)->GetPointCount();
	Vector *	vec			= ToPoint(cache)->GetPoint();
		
	for( i = 0; i < num_points; i++ )
		obj.createPoint( vec[i].x * config->scaleFactor, 
						 vec[i].y * config->scaleFactor, 
						 vec[i].z * config->scaleFactor );
	//////////////////////////////////////////////////////////////

	// Create Polygons	//////////////////////////////////////////

	LONG		num_polys	= ToPoly(cache)->GetPolygonCount();
	Polygon *	polys		= ToPoly(cache)->GetPolygon();
	

	for( i = 0; i < num_polys; i++ )
	{
		int ptIDs[4] = { polys[i].a, polys[i].b, polys[i].c, polys[i].d };
		
		if( ptIDs[2] == ptIDs[3] )
			obj.createPoly( 3, ptIDs, surf );
		else
			obj.createPoly( 4, ptIDs, surf );
	}
	//////////////////////////////////////////////////////////////
	
/*
	if( static_cast<mC4Dmhobject &>(obj).buildFlag )
	{
		obj.finalize( static_cast<mC4Dmhobject &>(obj).buildFlag );
		static_cast<mC4Dmhobject &>(obj).buildFlag = 0;
	}
	else
		obj.finalize( flags );

*/ // can't remember why I put this in, doesn't appear to do anything - ron 7/12

	obj.finalize( flags );	// DEBUG: flags should be 1 for new objects, and 0 for objects being loaded

//	static_cast<mC4Dmhobject &>(obj).updateMe = false;

	return true;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Copy object's name to messiah (::geometryCopy)  /////////////////////////////////////////////////////////////
// added 8/13/03 - ron
bool messiahC4Dscene_events_handler::objectRename( mh::object &obj, int flags )
{
	BaseObject *c4dobj = static_cast<BaseObject*>(obj.d_data2);

	if( !c4dobj ) return false;
	
	String sname = c4dobj->GetName();
	char *name = new char[sname.GetCStringLen()+1];
	sname.GetCString( name, sname.GetCStringLen()+1 );

	obj.rename( name, flags );

	delete [] name;

	return true;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Construct a new object in c4d (::constructObject)  //////////////////////////////////////////////////////////
// added 8/13/03 - ron
void * messiahC4Dscene_events_handler::constructObject( mh::object &obj, int flags )
{
	PolygonObject *polyobj = PolygonObject::Alloc( obj.numPoints(), obj.numPolys() );
	
	if( !polyobj )
	{
		GePrint("messiahC4D: Failed to create host object");
		return NULL;
	}

	Vector *			pts = polyobj->GetPoint();
	Polygon *			polys = polyobj->GetPolygon();
	mh::point_info		pntinfo;
	mh::polygon_info	polyinfo;


	int i;

	// set point positions
	for( i = 0; i < obj.numPoints(); i++ )
	{
		obj.getFilePoint( i, pntinfo );

		pts[i].x = pntinfo.x / config->scaleFactor;
		pts[i].y = pntinfo.y / config->scaleFactor;
		pts[i].z = pntinfo.z / config->scaleFactor;

	}

	// assign points to polygons
	for( i = 0; i < obj.numPolys(); i++ )
	{
		obj.getPolygon( i, polyinfo );

		if( polyinfo.d_numPoints == 3 )
			polys[i] = Polygon( polyinfo.d_pointIDs[0], polyinfo.d_pointIDs[1], polyinfo.d_pointIDs[2] ); 
		else
			polys[i] = Polygon( polyinfo.d_pointIDs[0], polyinfo.d_pointIDs[1], polyinfo.d_pointIDs[2], polyinfo.d_pointIDs[3] ); 
	}

	// name the new object
	char name[256] = {0};
	obj.getName( name, 256 );

	String sname(name);
	polyobj->SetName( sname );

	// insert the object into the document ( NOTE: since c4d can have multiple documents open, and messiah has only one global document
	// we must assume that the intended document in c4d is the active one )
	BaseDocument *doc = GetActiveDocument();
	if( !doc )
	{
		GePrint("messiahC4D: Failed to insert host object into active document");
		return NULL;
	}

	doc->InsertObject( polyobj, NULL, NULL, TRUE );
	doc->SetChanged();

	polyobj->SetDirty( DIRTY_DATA );

	return (void *)polyobj;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

mh::object * messiahC4Dscene_events_handler::addPlugin( mh::object &obj, void *data, int plug_type, int flags)
{
	if( !data ) return NULL;

	switch( plug_type )
	{
	case mh::object::Displacement:
		{
			BaseTag *tag = BaseTag::Alloc( ID_MESSIAH_DEFORM_TAG );
			if( !tag )
			{
				GePrint("messiahC4D: Failed to create new messiahDform tag");
				return NULL;
			}

			PluginObject *plugin = PluginObject::Alloc( ID_MESSIAH_DEFORM );
			if( !plugin )
			{
				GePrint("messiahC4D: Failed to create new messiahDform plugin object");
				return NULL;
			}

			BaseContainer *cont = tag->GetDataInstance();
			BaseObject *obj = (BaseObject *)data;

			mC4Dmhobject *mh_obj = new mC4Dmhobject;

			mh_obj->d_data1 = static_cast<void *>( obj );
			mh_obj->d_data2 = static_cast<void *>( obj );

			mh_obj->dirty = obj->GetDirty( DIRTY_DATA );

			BaseContainer newCont;

			newCont.SetLong( ID_MH_OBJ_PTR, reinterpret_cast<LONG>(mh_obj) );
			cont->SetContainer( ID_MESSIAH_DATA_ATTACH, newCont );

			// insert the plugin object into the document ( NOTE: since c4d can have multiple documents open, 
			// and messiah has only one global document
			// we must assume that the intended document in c4d is the active one )
			BaseDocument *doc = GetActiveDocument();
			if( !doc )
			{
				GePrint("messiahC4D: Failed to insert host object into active document");
				return NULL;
			}

			doc->InsertObject( plugin, obj, NULL, TRUE );
			doc->SetChanged();

			mh::scene *scene = GetMessiahSceneFromDocument( *GetActiveDocument() );
			scene->connectExportedObj( mh_obj );
				
			obj->InsertTag( tag );
			return mh_obj;
		}
		break;

	case mh::object::Motion:
		{
			BaseTag *tag = BaseTag::Alloc( ID_MESSIAH_MOVER );
			if( !tag )
			{
				GePrint("messiahC4D: Failed to create new messiahXform tag");
				return NULL;
			}

			BaseContainer *cont = tag->GetDataInstance();
			BaseObject *obj = (BaseObject *)data;

			mC4Dmhobject *mh_obj = new mC4Dmhobject;

			mh_obj->d_data1 = static_cast<void *>( obj );
			mh_obj->d_data2 = static_cast<void *>( obj );

			mh_obj->dirty = obj->GetDirty( DIRTY_DATA );

			BaseContainer newCont;

			newCont.SetLong( ID_MH_OBJ_PTR, reinterpret_cast<LONG>(mh_obj) );
			cont->SetContainer( ID_MESSIAH_DATA_ATTACH, newCont );
		
			mh::scene *scene = GetMessiahSceneFromDocument( *GetActiveDocument() );
			scene->connectExportedObj( mh_obj );
				
			obj->InsertTag( tag );
			return mh_obj;
		}
		break;
	}

	return NULL;
}