#include "messiahC4Dclasses.h"
#include "messiahC4Dglobals.h"
#include "messiahC4Dmhobject.h"
#include "messiahSceneHook.h"

#include <stdio.h>
#include <queue>

// old config file code, temporary
#include "ConfigData.h"

// Globals	/////////////////////////////////////////////////////////////////////////////////////////////////

extern mh::module *		messiahHost;
extern char				progName[];
extern char				plugName[];
extern ConfigData *		config;
extern bool				messiah_loaded;

// Default Ctor (::messiahDeformer)  ////////////////////////////////////////////////////////////////////////
messiahDeformer::messiahDeformer()
{
	// If messiah is not loaded, do so now
	// This will avoid the overhead of loading messiah until the user needs it
	if( messiah_loaded && !messiahHost->isMessiahActive() )
		if( !messiahHost->startup( progName, plugName ) )
		{
			GePrint("Error loading messiah! messiahC4D will not work properly!");
		}
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////


// Deform Points (::ModifyObject)  //////////////////////////////////////////////////////////////////////////

Bool messiahDeformer::ModifyObject(PluginObject* mod, BaseDocument* doc, BaseObject* op, 
					  const Matrix& op_mg, const Matrix& mod_mg, Real lod, LONG flags, BaseThread* thread)
{
	if( !messiah_loaded ) return FALSE;

	LONG num_points;
	Vector *vec;
	int i = 0;
	BaseObject *obj;
	BaseContainer *cont;
	BaseContainer *data;
	BaseTag *tag;
	mC4Dmhobject *mh_obj = NULL;	// derived from mh::object

	// Get the mh::scene from the scene hook
	mh::scene* docscene = GetMessiahSceneFromDocument(*doc);
	if (!docscene) return FALSE;

	// Only interested in point objects right now
	if(!op->IsInstanceOf(Opoint)) return TRUE;


	// Attempt to get link information, if none exists, create the link to messiah
	if( !( obj = GetPersistentObj( op )) ) return FALSE;

	// Only affect objects with dform tag applied
	if( !(tag = GetMessiahDformTag( obj )) ) return TRUE;

	if( !( cont = tag->GetDataInstance()) ) return FALSE;
	if( !( data = cont->GetContainerInstance( ID_MESSIAH_DATA_ATTACH)) )
	// This host object has not been linked, do so now
	{
		BaseContainer newCont;
		mh_obj = new mC4Dmhobject;

		// establish link between host plugin object and host object
		// this link will be re-established each time this method is called
		// the link will then be broken upon exit
		mh_obj->d_data1 = static_cast<void *>( op );

		// keep a link to the persistent object
		// NOTE: this object cannot be used to retrieve geom
		// added 8/12/03 - ron
		mh_obj->d_data2 = static_cast<void*>( obj );

		// establish link between host object and host plugin object
		newCont.SetLong( ID_MH_OBJ_PTR, reinterpret_cast<LONG>(mh_obj) );
		cont->SetContainer( ID_MESSIAH_DATA_ATTACH, newCont );

		// establish link between host plugin object and messiah object
		String sname = obj->GetName();
		char *name = new char[sname.GetCStringLen()+1];
		sname.GetCString( name, sname.GetCStringLen()+1 );
		
		docscene->connectExistingObj( mh_obj, name );
		mh_obj->setPluginType( mh::object::Displacement );
		// replaced 7/12 - ron 
		// messiahHost->getScene()->connectExistingObj( mh_obj, name );

		// store update value
		mh_obj->dirty = obj->GetDirty(DIRTY_DATA);
	}
	else
	// This host object is already linked, get the mh::object from it
	{
		mh_obj = reinterpret_cast<mC4Dmhobject *>(data->GetLong( ID_MH_OBJ_PTR ));
		if(!mh_obj) return FALSE;

		if( mh_obj->loading )
		{
			if( !docscene->isLoading() ) // scene is done loading, object is connected, now rebuild the mesh
			{
				mh_obj->loading = false;
				
				// establish link between host plugin object and host object
				// this link will be re-established each time this method is called
				// the link will then be broken upon exit
				mh_obj->d_data1 = static_cast<void *>( op );

				// keep a link to the persistent object
				// NOTE: this object cannot be used to retrieve geom
				// added 8/12/03 - ron
				mh_obj->d_data2 = static_cast<void*>( obj );
				
				char const * dbg_name = mh_obj->name();
				mh_obj->rebuild(0);
				dbg_name = mh_obj->name();
				mh_obj->dirty = obj->GetDirty(DIRTY_DATA);
			}
		}
		
		// Check to see if the host object's topology has changed, if so rebuild the obj
		if( obj->GetDirty(DIRTY_DATA) != mh_obj->dirty )
		{
			mh_obj->d_data1 = static_cast<void *>( op );
			mh_obj->d_data2 = static_cast<void*>( obj );
			mh_obj->rebuild(0);
			mh_obj->dirty = obj->GetDirty(DIRTY_DATA);
		}
	}

	num_points = ToPoint(op)->GetPointCount();
	vec = ToPoint(op)->GetPoint();

	if(!num_points)
		GePrint("No Points to deform");
		
	// Loop through points
	for( i = 0; i < num_points; i++ )
	{
		mh::point_info	pti;

		if( !mh_obj->getPoint( i, pti ) )
			GePrint("No messiah points to deform");
		
		vec[i].x = pti.x / config->scaleFactor;
		vec[i].y = pti.y / config->scaleFactor;
		vec[i].z = pti.z / config->scaleFactor;
	}

	// We can't hang onto op, and have no need for it outside of this method anyway
	mh_obj->d_data1 = NULL;

	return TRUE;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Update Objects (::CheckDirty)  //////////////////////////////////////////////////////////////////////////

void messiahDeformer::CheckDirty(PluginObject *op, BaseDocument *doc )
{
	if ( doc->GetTime()!= this->lastTime )
	{
		this->lastTime = doc->GetTime();
		op->SetDirty(DIRTY_DATA);
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Save Plugin (::Write)  //////////////////////////////////////////////////////////////////////////

Bool messiahDeformer::Write(GeListNode *node, HyperFile *hf)
{

	if( messiah_loaded && !messiahHost->getScene()->isSaving() )
	{
		messiahHost->getScene()->prepareForSave();
	}

	return TRUE;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Load Plugin (::Read)  /////////////////////////////////////////////////////////////////////////////////////////

Bool messiahDeformer::Read(GeListNode *node, HyperFile *hf, LONG level )
{

	if( messiah_loaded && !messiahHost->getScene()->isLoading() )
	{
		messiahHost->getScene()->prepareForLoad();
	}

	return TRUE;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Register Messiah Deformer Plugin  //////////////////////////////////////////////////////////////////////////

Bool RegisterMessiahDeformer(void)
{
	String name;
	name = GeLoadString(IDS_MESSIAH_DEFORMER);

	if( !RegisterObjectPlugin( ID_MESSIAH_DEFORM, name, OBJECT_MODIFIER, 
		 messiahDeformer::Alloc,"", "messiah.bmp", "messiah_small.bmp", 0) )	return FALSE;
  
	return TRUE;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

