// ==================		Params.h				==============
//
// Copyright (C) 2002 pmG Worldwide LLC. All Rights Reserved.
//
// ===============================================================
//
// Description: 
//
// This file contains all global paramaters such as global variables
// and macros used throughout this project.
//
// ===============================================================
//
// Version History:
//
// Written By: Ron Griswold
// Date: 4/18/2002
//
// Revised By:
// Date: 
//
// ===============================================================

#ifndef MM_PARAMS
#define MM_PARAMS

#include "Misc.h"

// ===============================================================
//			Manager Side
// ===============================================================
#include "messiahMayaManager.h"

// ===============================================================
//			Messiah Side
// ===============================================================
#include "MH_System.h"
#include "MH_NodeObject.h"
#include "MH_NodeLogFuncs.h"

#ifdef MESSIAH_MAYA_VERSION_8 //Dan 8-8-03
char const * const HOST_APP_NAME	= "Maya 8.0";
char const * const HOST_MODULE_NAME	= "messiahMaya80.mll";
#else
	#ifdef MESSIAH_MAYA_VERSION_7 //Dan 8-8-03
	char const * const HOST_APP_NAME	= "Maya 7.0";
	char const * const HOST_MODULE_NAME	= "messiahMaya70.mll";
	#else
		#ifdef MESSIAH_MAYA_VERSION_65 //Dan 8-8-03
		char const * const HOST_APP_NAME	= "Maya 6.5";
		char const * const HOST_MODULE_NAME	= "messiahMaya65.mll";
		#else
			#ifdef MESSIAH_MAYA_VERSION_6 //Dan 8-8-03
			char const * const HOST_APP_NAME	= "Maya 6.0";
			char const * const HOST_MODULE_NAME	= "messiahMaya60.mll";
			#else
				#ifdef MESSIAH_MAYA_VERSION_5 //Dan 8-8-03
				char const * const HOST_APP_NAME	= "Maya 5.0";
				char const * const HOST_MODULE_NAME	= "messiahMaya50.mll";
				#else
					#ifdef MESSIAH_MAYA_VERSION_45 //Dan 3-28-03
					char const * const HOST_APP_NAME	= "Maya 4.5";
					char const * const HOST_MODULE_NAME	= "messiahMaya45.mll";
					#else
						#ifdef MESSIAH_MAYA_VERSION_4 //Dan 3-28-03
						char const * const HOST_APP_NAME	= "Maya 4.0";
						char const * const HOST_MODULE_NAME	= "messiahMaya40.mll";
						#else
						char const * const HOST_APP_NAME	= "Maya";
						char const * const HOST_MODULE_NAME	= "messiahMaya.mll";
						#endif
					#endif
				#endif
			#endif
		#endif
	#endif
#endif
// ===============================================================
//			Maya Side
// ===============================================================

#include <maya/MTime.h>

// Global Variables
static MTime	MM_MayaCurrentTime;

// Node ID no.s
int	const MM_DISP_ID				= 0x80000;		// <<=== Must be replaced by valid id before distribution
int const MM_MOVE_ID				= 0x80001;		// <<=== Must be replaced by valid id before distribution
int const MM_MAN_ID					= 0x80002;		// <<=== Must be replaced by valid id before distribution

int const MM_MAX_POINTS_PER_POLY	= 256;

// Plugin Registration
char const * const MM_VENDOR				= "pmG";
char const * const MM_PLUGIN_VERSION		= "1.0 beta1";
char const * const MM_API_VERSION			= "Any";

// Nodes
char const * const MM_DISP_NAME				= "messiahDform";
char const * const MM_MOVE_NAME				= "messiahXform";
char const * const MM_MAN_NAME				= "messiahMaya";
	
	// attribs common
	char const * const MM_ATR_TIME_S			= "ti";
	char const * const MM_ATR_TIME_L			= "time";
	char const * const MM_ATR_HOST_ITEM_S		= "hi";
	char const * const MM_ATR_HOST_ITEM_L		= "hostItem";
	char const * const MM_ATR_LOAD_ID_S			= "lid";
	char const * const MM_ATR_LOAD_ID_L			= "loadId";

	// attribs mover specific
	char const * const MM_ATR_ROTATE_S			= "r";
	char const * const MM_ATR_ROTATE_L			= "rotate";
	char const * const MM_ATR_ROTATE_X_S		= "rx";
	char const * const MM_ATR_ROTATE_X_L		= "rotateX";
	char const * const MM_ATR_ROTATE_Y_S		= "ry";
	char const * const MM_ATR_ROTATE_Y_L		= "rotateY";
	char const * const MM_ATR_ROTATE_Z_S		= "rz";
	char const * const MM_ATR_ROTATE_Z_L		= "rotateZ";

	char const * const MM_ATR_ROTATE_ORDER_S	= "ro";
	char const * const MM_ATR_ROTATE_ORDER_L	= "rotateOrder";

	char const * const MM_ATR_TRANS_S			= "t";
	char const * const MM_ATR_TRANS_L			= "translate";
	char const * const MM_ATR_TRANS_X_S			= "tx";
	char const * const MM_ATR_TRANS_X_L			= "translateX";
	char const * const MM_ATR_TRANS_Y_S			= "ty";
	char const * const MM_ATR_TRANS_Y_L			= "translateY";
	char const * const MM_ATR_TRANS_Z_S			= "tz";
	char const * const MM_ATR_TRANS_Z_L			= "translateZ";

	// attribs deform specific
	/* none */

	// attribs sceneNode specific
	char const * const MM_ATR_FXSBUFF_S			= "fxs";
	char const * const MM_ATR_FXSBUFF_L			= "fxsBuffer";
	char const * const MM_ATR_SCALE_S			= "sf";
	char const * const MM_ATR_SCALE_L			= "scaleFactor";

// Commands
char const * const MM_CMD_NAME					= "messiah";

	// args
	char const * const MM_ARG_TEST				= "-test";

	char const * const MM_ARG_HELP_S			= "-h";
	char const * const MM_ARG_HELP_L			= "-help";

	char const * const MM_ARG_ATTACH_XFORM_S	= "-ax";
	char const * const MM_ARG_ATTACH_XFORM_L	= "-attachXform";

	char const * const MM_ARG_ATTACH_DFORM_S	= "-ad";
	char const * const MM_ARG_ATTACH_DFORM_L	= "-attachDform";

	char const * const MM_ARG_SET_SCALE_S		= "-sf";
	char const * const MM_ARG_SET_SCALE_L		= "-scaleFactor";

	char const * const MM_ARG_QUERY_S			= "-q";
	char const * const MM_ARG_QUERY_L			= "-query";

#endif	// MM_PARAMS