// ========			messiahCommonNodeOps.cpp		==============
//
// Copyright (C) 2002 pmG Worldwide LLC. All Rights Reserved.
//
// ===============================================================
//
// Description: 
//
// Implementation of messiahCommonNodeOps.
//
// ===============================================================
//
// Version History:
//
// Written By: Ron Griswold
// Date: 4/18/2002
//
// Revised By:
// Date: 
//
// ===============================================================
#include "messiahCommonNodeOps.h"
#include "messiahTransformerNode.h"
#include "messiahDeformerNode.h"

#include <string.h>

#include <maya/MGlobal.h>
#include <maya/MCommandResult.h>

#include <maya/MPxNode.h>

#include <maya/MFnDagNode.h>
#include <maya/MFnDependencyNode.h>
#include <maya/MFnTransform.h>

#include <maya/MDagPath.h>
#include <maya/MTypeId.h> 
#include <maya/MDGModifier.h>
#include <maya/MDagModifier.h>

#include <maya/MPlug.h>
#include <maya/MPlugArray.h>

#include <maya/MFnMesh.h>
#include <maya/MFloatPointArray.h>
#include <maya/MFloatPoint.h>
#include <maya/MPoint.h>
#include <maya/MPointArray.h>
#include <maya/MIntArray.h>

#include <maya/MItGeometry.h>

int		common::hostObjectGeometryCopy		(FX_NodeLog *messiahNode, int addToHierarchy)
{
	
	if( !messiahNode )
		return FALSE;

	MStatus		stat;

	MObject		transform;
	MObject		shape;
	
	MPlug				hiMsg;
	MPlugArray			plugArray;
	MPlug				plug;
	MFnDependencyNode	fnDN;
	MFnDagNode			dag;
	MDagPath			path;
	
	MPxNode *	plugin = (MPxNode *)messiahNode->element;
	if(!plugin) return FALSE;

	// ===============================================================
	// Xform node
	// ===============================================================
	if( messiahNode->type == FX_NODE_MOTION )
	{
		// ===============================================================
		// traverse the connections to get to attached host item
		// ===============================================================
		fnDN.setObject( plugin->thisMObject() );
		
		// get host item plug from node
		plug = fnDN.findPlug( MString(MM_ATR_HOST_ITEM_S) );
		if( !plug.isConnected() )
			return FALSE;

		// get connected plug
		plug.connectedTo( plugArray, true, false );
		hiMsg = plugArray[0];

		// get host item node
		transform = hiMsg.node();
		dag.setObject( transform );
		dag.getPath( path );
		path.extendToShape();
		shape = path.node();

		// store shape path in instance data
		messiahTransformer * xnode = (messiahTransformer *)plugin;
		xnode->_shape = path;

		// ===============================================================

		// ===============================================================
		// add geometry if nescessary
		// ===============================================================
		int tmp = shape.apiType();
		switch( shape.apiType() )
		{
		case MFn::kLocator:
			{
				MH_NodeObject no( messiahNode->obj );

				common::copyLocatorToMessiah(no, shape, addToHierarchy);
			}
			break;

		case MFn::kMesh:
			{
				MH_NodeObject	no( messiahNode->obj );

				common::copyMeshToMessiah(no, shape, addToHierarchy);
			}
			break;

		case MFn::kNurbsSurface:
			{

			}
			break;

		case MFn::kSubdiv:
			{

			}
			break;

		}// switch
	}// if messiahXform

	// ===============================================================
	// Dform node
	// ===============================================================
	else if( messiahNode->type == FX_NODE_DISPLACEMENT )
	{
		MObject current = plugin->thisMObject();
		MPlug outputPlug;

		// ===============================================================
		// trace nodes upstream until a geom node is reached
		// ===============================================================
		while( true ) // break when a recognized node type is reached, or no more inputGeom connections
		{
			stat = fnDN.setObject( current );
			if( !stat ) return FALSE;

			plug = fnDN.findPlug( MString("inputGeometry"), &stat );
			if( !stat ) return FALSE;

			if( !plug.isConnected() ) return FALSE;

			if( !plug.connectedTo( plugArray, true, false ) ) return FALSE;
			outputPlug = plugArray[0];

			// get host item node
			current = outputPlug.node();

			if( current.apiType() == MFn::kLocator ||
				current.apiType() == MFn::kMesh ) break;
		}

		// store shape path in instance data
		MDagPath shapePath;
		MFnDagNode	dag( current );
		dag.getPath( shapePath );

		messiahDeformer * xnode = (messiahDeformer *)plugin;
		xnode->_shape = shapePath;

		// ===============================================================
		// add geometry if nescessary
		// ===============================================================
		switch( current.apiType() )
		{
		case MFn::kLocator:
			{
				MH_NodeObject no( messiahNode->obj );

				common::copyLocatorToMessiah(no, current, addToHierarchy);
			}
			break;

		case MFn::kMesh:
			{
				MH_NodeObject	no( messiahNode->obj );

				common::copyMeshToMessiah(no, current, addToHierarchy);
			}
			break;

		case MFn::kNurbsSurface:
			{

			}
			break;

		case MFn::kSubdiv:
			{

			}
			break;

		}// switch

	}
	
	return TRUE;
}


int		common::hostObjectGetPoints			(FX_NodeLog *messiahNode, int resetObject)
{
	MPxNode *				node = NULL;
	messiahDeformer *		dnode = NULL;
	messiahTransformer *	xnode = NULL;
	commonNode *			cnode = NULL;

	if(!messiahNode)
		return FALSE;

	if( !(node = (MPxNode *)messiahNode->element) )
		return FALSE;

	if( node->typeId() == MM_MOVE_ID )
	{
		xnode = (messiahTransformer *)node;
		cnode = (commonNode *)xnode;
	}
	else if( node->typeId() == MM_DISP_ID )
	{
		dnode = (messiahDeformer *)node;
		cnode = (commonNode *)dnode;
	}

	if( cnode->_shape.isValid() )
	{
		MItGeometry	it( cnode->_shape );

		MH_NodeObject no( messiahNode->obj );

		for( int i = 0; i < it.count() && !it.isDone(); i++, it.next() )
		{
			MPoint pt = it.position();
			no.SetPoint(i, -pt[0]*MH_System::scaleFactor, pt[1]*MH_System::scaleFactor, pt[2]*MH_System::scaleFactor);	// invert X for handedness conversion
		}

		return TRUE;
	}

	return FALSE;
}

int		common::hostObjectRename			(FX_NodeLog *messiahNode, int flags)
{

	// TODO

	return TRUE;
}

/*
void*		common::hostObjectAdd				(void *obj, char *name, int type, int sub_type, int flags)
{
	MStatus				stat;
	MH_NodeObject		no(obj);
	
	MFnDependencyNode	dnode;
	MFnDagNode			dag;
	MFnTransform		tnode;
	MDagModifier		mDagMod;
	MObject				xobj;
	char				mobjName[256];

	// get object name from messiah
	no.GetName(mobjName, 256);

	MString			resStr, cmd;
	MCommandResult	res;

	cmd = "CreateLocator";
	stat = MGlobal::executeCommand(cmd, resStr);
	//res.getResult( resStr );
	

	// get transform node's name to pass to hostAddPlugin
	char *ret = new char[256]; // deleted in hostAddPlugin

	strcpy(ret, resStr.asChar() );

	return (void *)(ret);

}
*/


void *	common::hostObjectConstruct			(int obj, int flags)
{
	MStatus stat;

	puts("called hostObjectConstruct");
	MH_NodeObject no(obj);
	
	int	numPoints = 0;
	int numPolys = 0;
	
	double	x, y, z;

	int		ptIDs[MM_MAX_POINTS_PER_POLY];
	int		numMessiahPoints;
	int		idMessiahSurface;

	int i, j;

	MString			resStr, cmd;
	MString			seName, shaderName;
	MCommandResult	res;
	MString			objName;

	MFnDependencyNode	dnode;

	// get point data from messiah
	numPoints = nodeObjectNumPoints(obj); //no.NumPoints();
	
	MPointArray	pts(numPoints);

	for(i = 0; i < numPoints; i++)
	{
		no.GetPoint( i, &x, &y, &z);
		pts.set(i, -x/MH_System::scaleFactor, y/MH_System::scaleFactor, z/MH_System::scaleFactor);
	}

	// get poly data from messiah
	numPolys = no.NumPolys();

	MIntArray ptcnts(numPolys);
	MIntArray connects;
	unsigned int lentmp = 0;

	for(i = 0; i < numPolys; i++)
	{
		memset(ptIDs, 0, sizeof(int)*MM_MAX_POINTS_PER_POLY);

		no.GetPoly(i, ptIDs, MM_MAX_POINTS_PER_POLY, &numMessiahPoints, &idMessiahSurface);
		ptcnts.set(numMessiahPoints, i);

		for(j = numMessiahPoints - 1; j >= 0 ; j--)
			connects.append(ptIDs[j]);

		lentmp = connects.length();
	}

	// add new item to Maya
	MFnMesh	mesh;
	MObject xobj, mobj;
	char mobjName[256];
		
	// get object name from messiah
	no.GetName(mobjName, 256);

	// create transform node
	MDagModifier mDagMod;
	xobj = mDagMod.createNode("transform", MObject::kNullObj, &stat );
	mDagMod.renameNode( xobj, mobjName );
	mDagMod.doIt();

	// create mesh node
	mobj = mesh.create(numPoints, numPolys, pts, ptcnts, connects, xobj);
	mDagMod.renameNode( mobj, MString(mobjName) + "Shape");
	mDagMod.doIt();

	// attache to the shading engine
	dnode.setObject(mobj);
	objName = dnode.name();

	cmd = "connectAttr -na " + objName + ".instObjGroups initialShadingGroup.dagSetMembers";
	stat = MGlobal::executeCommand(cmd, res);

	// get transform node's name to pass to hostAddPlugin
	dnode.setObject(xobj);
	char *ret = new char[256]; // deleted in hostAddPlugin

	strcpy(ret, dnode.name().asChar() );

	return (void *)(ret);
}

extern int temporary_messiahobj_global;	//messiahCommand.cpp
	
int			common::hostAddPlugin	(void *obj_data, int obj, int plugType, int flags)
{
	if(!obj_data || !obj) return 0;

	char *name = (char *)obj_data;

	// select the maya object
	MGlobal::selectByName(name, MGlobal::ListAdjustment::kReplaceList);

	temporary_messiahobj_global = obj;

	MString			cmdRes;
	MCommandResult	res;

	if( plugType == FX_NODE_DISPLACEMENT )
	MGlobal::executeCommand("messiah -ad", res);
	else if ( plugType == FX_NODE_MOTION )
		MGlobal::executeCommand("messiah -ax", res);

	res.getResult(cmdRes);
	MGlobal::displayInfo(cmdRes);

	delete [] name;
	temporary_messiahobj_global = 0;

	return 1;
}

// ===============================================================
// Host Global
// ===============================================================
#include <stdio.h>
#include <string.h>

#define	HOST_SET(v)			if (!strcmp(#v,globalName)) return &##v

void *	common::hostGlobal	(char *globalName )
{
	std::string	global(globalName);

	puts(globalName);

	if( global == "hostObjectGeometryCopy" )
	{
		return (void *)common::hostObjectGeometryCopy;
	}
	if( global == "hostObjectGetPoints" )
	{
		return (void *)common::hostObjectGetPoints;
	}
	if( global == "hostObjectRename" )
	{
		return (void *)common::hostObjectRename;
	}
	if( global == "hostObjectConstruct" )
	{
		return (void *)&common::hostObjectConstruct;
	}
//	if( global == "hostObjectAdd" )
//	{
//		return (void *)&common::hostObjectAdd;
//	}

	if( global == "hostAddPlugin" )
	{
		return (void *)common::hostAddPlugin;
	}

	return NULL;
}

bool	common::copyLocatorToMessiah		(MH_NodeObject &no, MObject mayaItem, int addToHierarchy)
{
	no.Append( addToHierarchy );

	return true;
}

bool	common::copyMeshToMessiah			(MH_NodeObject &no, MObject mayaItem, int addToHierarchy)
{
	MFnMesh			mesh( mayaItem );
	int				numPoints = 0;
	int				numPolys  = 0;
	int				k = 0;
	int				l = 0;

	numPoints = mesh.numVertices();
	numPolys  = mesh.numPolygons();

	MFloatPointArray	pointArray;
	MFloatPoint			point;

	mesh.getPoints( pointArray );

	MIntArray		vertList;

	// create surface(s)
	// TODO get maya surfaces
	no.CreateSurface("mayaSurface");

	// copy the points from maya to messiah
	int i = 0;
	for(i = 0; i < numPoints; i++)
	{
		point = pointArray[i];

		no.CreatePoint( -point[0]*MH_System::scaleFactor, point[1]*MH_System::scaleFactor, point[2]*MH_System::scaleFactor );
	}// foreach point

	// copy the polys from maya to messiah
	for(i = 0; i < numPolys; i++)
	{	
		mesh.getPolygonVertices( i, vertList );

		int vertCount = mesh.polygonVertexCount(i);

		// copy the vertList to an int array
		int *		intArray = NULL;
		int *		flipArray = NULL;
		intArray = new int[ vertList.length() ];
		flipArray = new int[ vertList.length() ];

		for( int j = 0; j < vertCount; j++)
			intArray[j] = vertList[j];
//		for( int j = vertCount, k = 0; j > 0; j--, k++)
//			intArray[k] = vertList[j];

		for( k = 0, l = vertList.length() - 1; k < vertList.length(); k++, l--)
			flipArray[k] = intArray[l];

		no.CreatePoly( vertCount, flipArray, 1, 0); // surface ID is temporary

		delete [] intArray;
		delete [] flipArray;

	}// foreach polygon


	no.End(addToHierarchy);


	return true;
}

bool	copyNurbsToMessiah			(MH_NodeObject &no, MObject mayaItem, int addToHierarchy)
{
	// could be lots-o-work

	return true;
}

bool	copySubdivToMessiah			(MH_NodeObject &no, MObject mayaItem, int addToHierarchy)
{
	// looks like it'll be more work than it should be

	return true;
}

