// ==========		messiahTransformerNode.cpp			==========
//
// Copyright (C) 2002 pmG Worldwide LLC. All Rights Reserved.
//
// ===============================================================
//
// Description: 
//
// Implementation of messianTransformer.
//
// ===============================================================
//
// Version History:
//
// Written By: Ron Griswold
// Date: 4/18/2002
//
// Revised By:
// Date: 
//
// ===============================================================

#include "messiahTransformerNode.h"

#include <maya/MFnNumericAttribute.h>
#include <maya/MFnMessageAttribute.h>
#include <maya/MFnUnitAttribute.h>

#include <maya/MFnDagNode.h>
#include <maya/MFnDependencyNode.h>

#include <maya/MDagPath.h>
#include <maya/MTypeId.h> 
#include <maya/MTime.h>

#include <maya/MPlug.h>
#include <maya/MPlugArray.h>
#include <maya/MDataBlock.h>
#include <maya/MDataHandle.h>

#include <maya/MGlobal.h>


// ===============================================================
// Initialize node type id
// ===============================================================
MTypeId     messiahTransformer::id( MM_MOVE_ID );

// ===============================================================
// Initialize attribs to NULL
// ===============================================================
MObject		messiahTransformer::hostItem;
MObject		messiahTransformer::loadId;

MObject		messiahTransformer::time;  

MObject		messiahTransformer::rotateX;
MObject		messiahTransformer::rotateY;
MObject		messiahTransformer::rotateZ;

MObject		messiahTransformer::translateX;
MObject		messiahTransformer::translateY;
MObject		messiahTransformer::translateZ;

MObject		messiahTransformer::rotateOrder;

// ===============================================================
// Constructor
// ===============================================================
messiahTransformer::messiahTransformer() 
{
	// Add the node to the list of nodes in the manager
	// See messiahMayaManager.h
	messiahMayaManager::Inst()->AddNode( this );
}

// ===============================================================
// Destructor
// ===============================================================
messiahTransformer::~messiahTransformer() 
{
	// Remove the node from the list of nodes
	// See above
	messiahMayaManager::Inst()->RemoveNode( this );
}

// ===============================================================
// Creator
// ===============================================================
void* messiahTransformer::creator()
{
	return new messiahTransformer();
}

// ===============================================================
// Initialize
// ===============================================================
MStatus messiahTransformer::initialize()
{
	MFnNumericAttribute		nAttr;
	MFnUnitAttribute		uAttr;
	MFnMessageAttribute		mAttr;

	MStatus					stat;

	hostItem = mAttr.create( MM_ATR_HOST_ITEM_L, MM_ATR_HOST_ITEM_S, &stat);
	mAttr.setReadable(false);

	loadId = nAttr.create( MM_ATR_LOAD_ID_L, MM_ATR_LOAD_ID_S, MFnNumericData::kLong );
	nAttr.setStorable( true );
	nAttr.setHidden( true );

	time = uAttr.create( MM_ATR_TIME_L, MM_ATR_TIME_S, MTime( 0.0 ), &stat);
	uAttr.setReadable( false );
	uAttr.setKeyable( false );

	rotateX = uAttr.create( MM_ATR_ROTATE_X_L, MM_ATR_ROTATE_X_S, MFnUnitAttribute::kAngle, 0.0, &stat );
	uAttr.setStorable( false );
	uAttr.setWritable( false );
	uAttr.setKeyable( false );

	rotateY = uAttr.create( MM_ATR_ROTATE_Y_L, MM_ATR_ROTATE_Y_S, MFnUnitAttribute::kAngle, 0.0, &stat );
	uAttr.setStorable( false );
	uAttr.setWritable( false );
	uAttr.setKeyable( false );

	rotateZ = uAttr.create( MM_ATR_ROTATE_Z_L, MM_ATR_ROTATE_Z_S, MFnUnitAttribute::kAngle, 0.0, &stat );
	uAttr.setStorable( false );
	uAttr.setWritable( false );
	uAttr.setKeyable( false );

	translateX = nAttr.create( MM_ATR_TRANS_X_L, MM_ATR_TRANS_X_S, MFnNumericData::kDouble, 0.0, &stat );
	nAttr.setStorable( false );
	nAttr.setWritable( false );
	nAttr.setKeyable( false );

	translateY = nAttr.create( MM_ATR_TRANS_Y_L, MM_ATR_TRANS_Y_S, MFnNumericData::kDouble, 0.0, &stat );
	nAttr.setStorable( false );
	nAttr.setWritable( false );
	nAttr.setKeyable( false );

	translateZ = nAttr.create( MM_ATR_TRANS_Z_L, MM_ATR_TRANS_Z_S, MFnNumericData::kDouble, 0.0, &stat );
	nAttr.setStorable( false );
	nAttr.setWritable( false );
	nAttr.setKeyable( false );

	rotateOrder = nAttr.create( MM_ATR_ROTATE_ORDER_L, MM_ATR_ROTATE_ORDER_S, MFnNumericData::kInt, 2, &stat);
	nAttr.setStorable( true );
	nAttr.setWritable( false );
	nAttr.setReadable( true );
	nAttr.setKeyable( false );
	nAttr.setHidden( true );

	stat = addAttribute( hostItem );
		if (!stat) { stat.perror("addAttribute"); return stat;}
	stat = addAttribute( loadId );
		if (!stat) { stat.perror("addAttribute"); return stat;}
	stat = addAttribute( time );
		if (!stat) { stat.perror("addAttribute"); return stat;}
	stat = addAttribute( rotateX );
		if (!stat) { stat.perror("addAttribute"); return stat;}
	stat = addAttribute( rotateY );
		if (!stat) { stat.perror("addAttribute"); return stat;}
	stat = addAttribute( rotateZ );
		if (!stat) { stat.perror("addAttribute"); return stat;}
	stat = addAttribute( translateX );
		if (!stat) { stat.perror("addAttribute"); return stat;}
	stat = addAttribute( translateY );
		if (!stat) { stat.perror("addAttribute"); return stat;}
	stat = addAttribute( translateZ );
		if (!stat) { stat.perror("addAttribute"); return stat;}
	stat = addAttribute( rotateOrder );
		if (!stat) { stat.perror("addAttribute"); return stat;}

	stat = attributeAffects( time, rotateX );
	stat = attributeAffects( time, rotateY );
	stat = attributeAffects( time, rotateZ );
	stat = attributeAffects( hostItem, rotateX );
	stat = attributeAffects( hostItem, rotateY );
	stat = attributeAffects( hostItem, rotateZ );
	stat = attributeAffects( time, translateX );
	stat = attributeAffects( time, translateY );
	stat = attributeAffects( time, translateZ );
	stat = attributeAffects( hostItem, translateX );
	stat = attributeAffects( hostItem, translateY );
	stat = attributeAffects( hostItem, translateZ );

	return MS::kSuccess;

}

// ===============================================================
// Compute
// ===============================================================
MStatus messiahTransformer::compute( const MPlug& plug, MDataBlock& data )
{
	MStatus					returnStatus;

	if( plug == rotateX )
	{
		MDataHandle r = data.outputValue( rotateX );
		MDataHandle frameH = data.inputValue( time );
		MTime frame = frameH.asTime();
		double frameDouble = frame.as(MTime::kFilm );

		double rot[3] = {0};

		MH_NodeObject no(this->_nodeLog->obj);
		no.GetWorldRot(rot, frameDouble );

		r.set( rot[1] );
		r.setClean();
	}
	else if( plug == rotateY )
	{
		MDataHandle r = data.outputValue( rotateY );
		MDataHandle frameH = data.inputValue( time );
		MTime frame = frameH.asTime();
		double frameDouble = frame.as(MTime::kFilm );

		double rot[3] = {0};

		MH_NodeObject no(this->_nodeLog->obj);
		no.GetWorldRot(rot, frameDouble );

		r.set( -rot[0] );
		r.setClean();
	}
	else if( plug == rotateZ )
	{
		MDataHandle r = data.outputValue( rotateZ );
		MDataHandle frameH = data.inputValue( time );
		MTime frame = frameH.asTime();
		double frameDouble = frame.as(MTime::kFilm );

		double rot[3] = {0};

		MH_NodeObject no(this->_nodeLog->obj);
		no.GetWorldRot(rot, frameDouble );

		r.set( -rot[2] );
		r.setClean();
	}
	else if( plug == translateX )
	{
		MDataHandle transOut = data.outputValue( translateX );
		MDataHandle frameH = data.inputValue( time );
		MTime frame = frameH.asTime();
		double frameDouble = frame.as( MTime::kFilm );

		double trans[3];

		MH_NodeObject no( this->_nodeLog->obj );
		no.GetWorldPos(trans, frameDouble );

		transOut.set( -trans[0]);
		transOut.setClean();
	}
	else if( plug == translateY )
	{
		MDataHandle transOut = data.outputValue( translateY );
		MDataHandle frameH = data.inputValue( time );
		MTime frame = frameH.asTime();
		double frameDouble = frame.as( MTime::kFilm );

		double trans[3];

		MH_NodeObject no( this->_nodeLog->obj );
		no.GetWorldPos(trans, frameDouble );

		transOut.set( trans[1]);
		transOut.setClean();
	}
	else if( plug == translateZ )
	{
		MDataHandle transOut = data.outputValue( translateZ );
		MDataHandle frameH = data.inputValue( time );
		MTime frame = frameH.asTime();
		double frameDouble = frame.as( MTime::kFilm );

		double trans[3];

		MH_NodeObject no( this->_nodeLog->obj );
		no.GetWorldPos(trans, frameDouble );

		transOut.set( trans[2]);
		transOut.setClean();
	}
	else
		return MS::kUnknownParameter;
 
	return MS::kSuccess;
}

extern int temporary_messiahobj_global;	//messiahCommand.cpp

// ===============================================================
// Connection Made
// ===============================================================
MStatus	messiahTransformer::connectionMade(const MPlug &plug, const MPlug &otherPlug, bool asSrc )
{
	MStatus stat;

	// ===============================================================
	// plug was connected to hostItem
	// ===============================================================
	if( plug == hostItem )
	{
		// Get the name of the host item ////////////////////////
		MObject			SourceObject = otherPlug.node();
		MFnDagNode		SourceDagNode( SourceObject );
		MDagPath		SourceDagPath;

		SourceDagNode.getPath( SourceDagPath );
		MString			PathName = SourceDagPath.fullPathName();
		char const *	tmp = PathName.asChar();
		char *			SourceName = new char[::strlen(tmp)+1];
		::strcpy( SourceName, tmp );
		
		MakeValidMessiahName( SourceName );
		/////////////////////////////////////////////////////////

		// Create the FX_NodeLog ////////////////////////////////
		this->_nodeLog = MH_NodeLogFuncs::LogElement(NULL, (void *)this, NULL, NULL);
		this->_nodeLog->type = FX_NODE_MOTION;
		/////////////////////////////////////////////////////////

		// Add item to messiah //////////////////////////////////
		// only if not scene loading ////////////////////////////
		if( ! (MAN::Inst()->SceneState() & MAN::SceneLoading) )
		{
			if( !temporary_messiahobj_global )
			{
				this->_nodeLog->obj = (MH_NodeObject::Create( SourceName, 0, 0 ))->MessiahObj();
				common::hostObjectGeometryCopy( this->_nodeLog, 1 );
			}
			else
			{
				this->_nodeLog->obj = temporary_messiahobj_global;
				temporary_messiahobj_global = 0;
			}
		}
		else
		{
			MPlug				loadId;
			int					id;
			MFnDependencyNode	dNode( this->thisMObject() );

			loadId = dNode.findPlug(MM_ATR_LOAD_ID_L, &stat);
			if(!stat)
			{
				MGlobal::displayError("Error Loading messiahTransformer");
				return MS::kFailure;
			}

			loadId.getValue( id );
			this->_nodeLog->load_ID = id;
		}
		/////////////////////////////////////////////////////////

	}

	// gotta return kUnknownParameter to force default handler to do it's thing
	// bad things happen if you return kSuccess
	return MS::kUnknownParameter;
}

