#include "ge_lvector.h"
#include "ge_lmatrix.h"

#include "c4d_file.h"
#include "c4d_memory.h"
#include "c4d_basecontainer.h"
#include "c4d_basebitmap.h"
#include "c4d_basechannel.h"
#include "c4d_general.h"

Filename::Filename(void)
{
	fn = C4DOS.Fn->Alloc();
	if (!fn) fn=C4DOS.Fn->Default;
}

Filename::Filename(const String &string)
{
	fn = C4DOS.Fn->Alloc();
	if (!fn) { fn=C4DOS.Fn->Default; return; }
	if (!string.Get()) return;
	C4DOS.Fn->SetString(fn,string.Get());
}

Filename::Filename(const Filename &src)
{
	fn = C4DOS.Fn->GetClone(src.fn);
	if (!fn) fn=C4DOS.Fn->Default;
}

Filename::Filename(_Filename *xn, Bool preallocated)
{
	if (preallocated || !xn)
		fn = xn;
	else
		fn = C4DOS.Fn->GetClone(xn);

	if (!fn) fn=C4DOS.Fn->Default;
}

Filename::~Filename(void)
{
	if (fn==NULL || fn==C4DOS.Fn->Default) return;
	C4DOS.Fn->Free(fn);
	fn = NULL;
}

Filename *Filename::GetClone(void) const
{
	if (!fn) return NULL;
	Filename *df = gNew Filename(fn,FALSE);
	return df;
}

Bool Filename::FileSelect(LONG type, LONG flags, String *title)
{
	if (!fn) return FALSE;
	return C4DOS.Fn->FileSelect(fn,type,flags,title?title->Get():NULL);
}

Bool Filename::Content(void) const
{
	if (!fn) return FALSE;
	return C4DOS.Fn->Content(fn);
}

String Filename::GetString(void) const
{
	_String *str=C4DOS.Fn->GetString(fn);
	if (!str) return String();
	String gs(str,TRUE);
	return gs;
}

void Filename::SetString(const String &str)
{
	if (!fn || !str.Get()) return;
	C4DOS.Fn->SetString(fn,str.Get());
}

Filename Filename::GetDirectory(void) const
{
	return Filename(C4DOS.Fn->GetDirectory(fn),TRUE);
}

Filename Filename::GetFile(void) const
{
	return Filename(C4DOS.Fn->GetFile(fn),TRUE);
}

String Filename::GetFileString(void) const
{
	return GetFile().GetString();
}

void Filename::ClearSuffix(void)
{
	if (!fn) return;
	C4DOS.Fn->ClearSuffix(fn);
}

void Filename::SetSuffix(const String &str)
{
	if (!fn || !str.Get()) return;
	C4DOS.Fn->SetSuffix(fn,str.Get());
}

Bool Filename::CheckSuffix(const String &str) const
{
	if (!fn || !str.Get()) return FALSE;
	return C4DOS.Fn->CheckSuffix(fn,str.Get());
}

void Filename::SetDirectory(const Filename &str)
{
	if (!fn || !str.Get()) return;
	C4DOS.Fn->SetDirectory(fn,str.Get());
}

void Filename::SetFile(const Filename &str)
{
	if (!fn || !str.Get()) return;
	C4DOS.Fn->SetFile(fn,str.Get());
}

const Filename& Filename::operator = (const Filename &fname)
{
	if (!fname.fn) return *this;
	if (fn && fn!=C4DOS.Fn->Default)
		C4DOS.Fn->Free(fn);

	fn=C4DOS.Fn->GetClone(fname.fn);
	if (!fn) fn=C4DOS.Fn->Default;

	return *this;
}

const Filename operator + (const Filename &Str1,const Filename &Str2)
{
	Filename fn = Str1;
	if (fn.fn && Str2.fn)
		C4DOS.Fn->Add(fn.fn,Str2.fn);
	return fn;
}

const Filename& Filename::operator += (const Filename &Str)
{
	if (fn && Str.fn)
		C4DOS.Fn->Add(fn,Str.fn);
	return *this;
}

Bool Filename::operator == (const Filename &Str) const
{
	if (!fn || !Str.fn) return FALSE;
	return C4DOS.Fn->Compare(fn,Str.fn);
}

Bool Filename::operator != (const Filename &Str) const
{
	if (!fn || !Str.fn) return TRUE;
	return !C4DOS.Fn->Compare(fn,Str.fn);
}

BaseFile *AllocBaseFile(void)
{
	return C4DOS.Fl->Alloc();
}

void FreeBaseFile(BaseFile *&fl)
{
	C4DOS.Fl->Free(fl);
	fl=NULL;
}

Bool BaseFile::Open(const Filename &name, LONG mode, LONG error_dialog, LONG order, LONG type, LONG creator)
{
	if (!name.Get()) return FALSE;
	return C4DOS.Fl->Open(this,name.Get(),mode,error_dialog,order,type,creator);
}

Bool BaseFile::Close(void)
{
	return C4DOS.Fl->Close(this);
}

void BaseFile::SetOrder(LONG order)
{
	C4DOS.Fl->SetOrder(this,order);
}

LONG BaseFile::ReadBytes(void *data, LONG len, Bool just_try_it)
{
	return C4DOS.Fl->ReadBytes(this,data,len,just_try_it);
}

Bool BaseFile::WriteBytes(void *data, LONG len)
{
	return C4DOS.Fl->WriteBytes(this,data,len);
}

LONG BaseFile::TryReadBytes(void *data, LONG len)
{
	return C4DOS.Fl->ReadBytes(this,data,len,TRUE);
}

Bool BaseFile::Seek(LONG pos, LONG mode)
{
	return C4DOS.Fl->Seek(this,pos,mode);
}

LONG BaseFile::GetPosition(void)
{
	return C4DOS.Fl->GetPosition(this);
}

LONG BaseFile::GetLength(void)
{
	return C4DOS.Fl->GetLength(this);
}

LONG BaseFile::GetError(void)
{
	return C4DOS.Fl->GetError(this);
}

void BaseFile::SetError(LONG error)
{
	C4DOS.Fl->SetError(this,error);
}

Bool BaseFile::ReadChar (CHAR *v)
{
	return C4DOS.Fl->ReadChar(this,v);
}

Bool BaseFile::ReadUChar(UCHAR *v)
{
	return C4DOS.Fl->ReadUChar(this,v);
}

Bool BaseFile::ReadWord (WORD *v)
{
	return C4DOS.Fl->ReadWord(this,v);
}

Bool BaseFile::ReadUWord(UWORD *v)
{
	return C4DOS.Fl->ReadUWord(this,v);
}

Bool BaseFile::ReadLong (LONG  *v)
{
	return C4DOS.Fl->ReadLong(this,v);
}

Bool BaseFile::ReadULong(ULONG *v)
{
	return C4DOS.Fl->ReadULong(this,v);
}

Bool BaseFile::ReadReal (Real  *v)
{
	return C4DOS.Fl->ReadReal(this,v);
}

Bool BaseFile::ReadLReal(LReal *v)
{
	return C4DOS.Fl->ReadLReal(this,v);
}

Bool BaseFile::ReadString(String *v)
{
	LONG  len=0;
	CHAR	*c = NULL;

	if (!ReadLong(&len)) return FALSE;
	if (!len) { *v=String(); return TRUE; }

	c = (CHAR*) GeAlloc(sizeof(CHAR)*len);
	if (!c)
	{
		SetError(FILEERROR_MEMORY);
		return FALSE;
	}
	if (!ReadBytes(c,len))
	{
		GeFree(c);
		return FALSE;
	}

	v->SetCString(c,len-1);
	GeFree(c);

	return TRUE;
}

Bool BaseFile::ReadFilename(Filename *v)
{
	String str;
	if (!ReadString(&str)) return FALSE;
	v->SetString(str);
	return TRUE;
}

Bool BaseFile::ReadBool(Bool *v)
{
	CHAR c;
	if (!ReadChar(&c)) return FALSE;
	*v = c!=0;
	return TRUE;
}

Bool BaseFile::ReadTime(BaseTime *v)
{
	return ReadReal(&v->nominator) && ReadReal(&v->denominator);
}

Bool BaseFile::ReadVector(Vector *v)
{
	return ReadReal(&v->x) && ReadReal(&v->y) && ReadReal(&v->z);
}

Bool BaseFile::ReadLVector(LVector *v)
{
	return ReadLReal(&v->x) && ReadLReal(&v->y) && ReadLReal(&v->z);
}

Bool BaseFile::ReadMatrix(Matrix *v)
{
	return ReadVector(&v->off) && ReadVector(&v->v1) && ReadVector(&v->v2) && ReadVector(&v->v3);
}

Bool BaseFile::ReadLMatrix(LMatrix *v)
{
	return ReadLVector(&v->off) && ReadLVector(&v->v1) && ReadLVector(&v->v2) && ReadLVector(&v->v3);
}

Bool BaseFile::WriteBool(Bool v)
{
	return WriteChar(CHAR(v!=0));
}

Bool BaseFile::WriteTime(const BaseTime &v)
{
	return WriteReal(v.nominator) && WriteReal(v.denominator);
}

Bool BaseFile::WriteVector(const Vector  &v)
{
	return WriteReal(v.x) && WriteReal(v.y) && WriteReal(v.z);
}

Bool BaseFile::WriteLVector(const LVector &v)
{
	return WriteLReal(v.x) && WriteLReal(v.y) && WriteLReal(v.z);
}

Bool BaseFile::WriteMatrix(const Matrix  &v)
{
	return WriteVector(v.off) && WriteVector(v.v1) && WriteVector(v.v2) && WriteVector(v.v3);
}

Bool BaseFile::WriteLMatrix(const LMatrix  &v)
{
	return WriteLVector(v.off) && WriteLVector(v.v1) && WriteLVector(v.v2) && WriteLVector(v.v3);
}

Bool BaseFile::WriteString(const String &v)
{
	Bool ok;
	LONG len  = v.GetCStringLen()+1;
	CHAR *mem = (CHAR*) GeAlloc(sizeof(CHAR)*len);
	if (!mem)
	{
		SetError(FILEERROR_MEMORY);
		return FALSE;
	}

	v.GetCString(mem,len);
	ok = WriteLong(len) && WriteBytes(mem,len);
	GeFree(mem);

	return ok;
}

Bool BaseFile::WriteFilename(const Filename &v)
{
	return WriteString(v.GetString());
}

Bool BaseFile::WriteChar (CHAR v)
{
	return C4DOS.Fl->WriteChar(this,v);
}

Bool BaseFile::WriteUChar(UCHAR v)
{
	return C4DOS.Fl->WriteUChar(this,v);
}

Bool BaseFile::WriteWord (WORD v)
{
	return C4DOS.Fl->WriteWord(this,v);
}

Bool BaseFile::WriteUWord(UWORD v)
{
	return C4DOS.Fl->WriteUWord(this,v);
}

Bool BaseFile::WriteLong (LONG v)
{
	return C4DOS.Fl->WriteLong(this,v);
}

Bool BaseFile::WriteULong(ULONG v)
{
	return C4DOS.Fl->WriteULong(this,v);
}

Bool BaseFile::WriteReal(Real v)
{
	return C4DOS.Fl->WriteReal(this,v);
}

Bool BaseFile::WriteLReal(LReal v)
{
	return C4DOS.Fl->WriteLReal(this,v);
}

BrowseFiles *AllocBrowseFiles(const Filename &directory, Bool calcsize)
{
	if (!directory.Get()) return NULL;
	return C4DOS.Bf->Alloc(directory.Get(),calcsize);
}

void FreeBrowseFiles(BrowseFiles *&bf)
{
	C4DOS.Bf->Free(bf);
	bf=NULL;
}

Bool BrowseFiles::GetNext(void)
{
	return C4DOS.Bf->GetNext(this);
}

LONG BrowseFiles::GetSize(void)
{
	return C4DOS.Bf->GetSize(this);
}

Bool BrowseFiles::IsDir(void)
{
	return C4DOS.Bf->IsDir(this);
}

Filename BrowseFiles::GetFilename(void)
{
	return Filename(C4DOS.Bf->GetFilename(this),TRUE);
}

Bool GeFExist(const Filename &name, Bool isdir)
{
	if (!name.Get()) return FALSE;
	return C4DOS.Ge->FExist(name.Get(),isdir);
}

Bool GeSearcHyperFile(const Filename &directory, const Filename &name, Filename *found)
{
	if (!name.Get() || !directory.Get() || !found->Get()) return FALSE;
	return C4DOS.Ge->SearchFile(directory.Get(),name.Get(),found->Get());
}

Bool GeFKill(const Filename &name, Bool isdir)
{
	if (!name.Get()) return FALSE;
	return C4DOS.Ge->FKill(name.Get(),isdir);
}

Bool GeFCopyFile(const Filename &source, const Filename &dest, Bool overwrite)
{
	if (!source.Get() || !dest.Get()) return FALSE;
	return C4DOS.Ge->FCopyFile(source.Get(),dest.Get(),overwrite);
}

Bool GeFRename(const Filename &source, const Filename &dest)
{
	if (!source.Get() || !dest.Get()) return FALSE;
	return C4DOS.Ge->FRename(source.Get(),dest.Get());
}

Bool GeFCreateDir(const Filename &name)
{
	if (!name.Get()) return FALSE;
	return C4DOS.Ge->FCreateDir(name.Get());
}

Bool GeExecuteFile(const Filename &path)
{
	if (!path.Get()) return FALSE;
	return C4DOS.Ge->ExecuteFile(path.Get());
}

Bool GeExecuteProgram(const Filename &program, const Filename &img)
{
	if (!program.Get() || !img.Get()) return FALSE;
	return C4DOS.Ge->ExecuteProgram(program.Get(),img.Get());
}

Filename GeGetStartupPath(void)
{
	return Filename(C4DOS.Ge->GetStartupPath(),TRUE);
}

Filename GeGetPluginPath(void)
{
	extern CHAR path_storage[1024];
	return Filename(String(path_storage)).GetDirectory();
}

Bool HyperFile::ReadChar (CHAR *v)
{
	return C4DOS.Hf->ReadChar(this,v);
}

Bool HyperFile::ReadUChar(UCHAR *v)
{
	return C4DOS.Hf->ReadUChar(this,v);
}

Bool HyperFile::ReadWord (WORD  *v)
{
	return C4DOS.Hf->ReadWord(this,v);
}

Bool HyperFile::ReadUWord(UWORD *v)
{
	return C4DOS.Hf->ReadUWord(this,v);
}

Bool HyperFile::ReadLong (LONG  *v)
{
	return C4DOS.Hf->ReadLong(this,v);
}

Bool HyperFile::ReadULong(ULONG *v)
{
	return C4DOS.Hf->ReadULong(this,v);
}

Bool HyperFile::ReadReal (Real  *v)
{
	return C4DOS.Hf->ReadReal(this,v);
}

Bool HyperFile::ReadLReal(LReal *v)
{
	return C4DOS.Hf->ReadLReal(this,v);
}

Bool HyperFile::ReadBool(Bool *v)
{
	return C4DOS.Hf->ReadBool(this,v);
}

Bool HyperFile::ReadTime(BaseTime *v)
{
	return C4DOS.Hf->ReadTime(this,v);
}

Bool HyperFile::ReadVector(Vector *v)
{
	return C4DOS.Hf->ReadVector(this,v);
}

Bool HyperFile::ReadLVector(LVector *v)
{
	return C4DOS.Hf->ReadLVector(this,v);
}

Bool HyperFile::ReadMatrix(Matrix *v)
{
	return C4DOS.Hf->ReadMatrix(this,v);
}

Bool HyperFile::ReadLMatrix(LMatrix *v)
{
	return C4DOS.Hf->ReadLMatrix(this,v);
}

Bool HyperFile::ReadString(String *v)
{
	if (!v->Get()) return FALSE;
	return C4DOS.Hf->ReadString(this,v->Get());
}

Bool HyperFile::ReadFilename(Filename *v)
{
	if (!v->Get()) return FALSE;
	return C4DOS.Hf->ReadFilename(this,v->Get());
}

Bool HyperFile::ReadContainer(BaseContainer *v, Bool flush)
{
	if (!v->Get()) return FALSE;
	return C4DOS.Hf->ReadContainer(this,v->Get(),flush);
}

Bool HyperFile::ReadMemory(void **data, LONG *size)
{
	return C4DOS.Hf->ReadMemory(this,data,size);
}

Bool HyperFile::ReadImage(BaseBitmap *v)
{
	return C4DOS.Hf->ReadImage(this,v);
}

Bool HyperFile::ReadChannel(BaseChannel *&bc)
{
	BaseChannel *nc = AllocBaseChannel();
	if (!nc) return FALSE;

	FreeBaseChannel(bc);
	bc=nc;
	C4DOS.Ba->ReadData(this,bc);
	return TRUE;
}

Bool HyperFile::WriteChannel(BaseChannel *bc)
{
	if (!bc) return FALSE;
	C4DOS.Ba->WriteData(this,bc);
	return TRUE;
}

Bool HyperFile::WriteChar (CHAR v)
{
	return C4DOS.Hf->WriteChar(this,v);
}

Bool HyperFile::WriteUChar(UCHAR v)
{
	return C4DOS.Hf->WriteUChar(this,v);
}

Bool HyperFile::WriteWord (WORD v)
{
	return C4DOS.Hf->WriteWord(this,v);
}

Bool HyperFile::WriteUWord(UWORD v)
{
	return C4DOS.Hf->WriteUWord(this,v);
}

Bool HyperFile::WriteLong (LONG v)
{
	return C4DOS.Hf->WriteLong(this,v);
}

Bool HyperFile::WriteULong(ULONG v)
{
	return C4DOS.Hf->WriteULong(this,v);
}

Bool HyperFile::WriteReal(Real v)
{
	return C4DOS.Hf->WriteReal(this,v);
}

Bool HyperFile::WriteLReal(LReal v)
{
	return C4DOS.Hf->WriteLReal(this,v);
}

Bool HyperFile::WriteBool(Bool v)
{
	return C4DOS.Hf->WriteBool(this,v);
}

Bool HyperFile::WriteTime(const BaseTime &v)
{
	return C4DOS.Hf->WriteTime(this,v);
}

Bool HyperFile::WriteVector(const Vector &v)
{
	return C4DOS.Hf->WriteVector(this,v);
}

Bool HyperFile::WriteLVector(const LVector &v)
{
	return C4DOS.Hf->WriteLVector(this,v);
}

Bool HyperFile::WriteMatrix(const Matrix &v)
{
	return C4DOS.Hf->WriteMatrix(this,v);
}

Bool HyperFile::WriteLMatrix(const LMatrix &v)
{
	return C4DOS.Hf->WriteLMatrix(this,v);
}

Bool HyperFile::WriteString(const String &v)
{
	if (!v.Get()) return FALSE;
	return C4DOS.Hf->WriteString(this,v.Get());
}

Bool HyperFile::WriteFilename(const Filename &v)
{
	if (!v.Get()) return FALSE;
	return C4DOS.Hf->WriteFilename(this,v.Get());
}

Bool HyperFile::WriteImage(BaseBitmap *v, LONG format, BaseContainer *data)
{
	return C4DOS.Hf->WriteImage(this,v,format,data?data->Get():NULL);
}

Bool HyperFile::WriteContainer(const BaseContainer &v)
{
	if (!v.Get()) return FALSE;
	return C4DOS.Hf->WriteContainer(this,v.Get());
}

Bool HyperFile::WriteMemory(void *data, LONG count)
{
	return C4DOS.Hf->WriteMemory(this,data,count);
}

LONG HyperFile::GetError(void)
{
	return C4DOS.Hf->GetError(this);
}

void HyperFile::SetError(LONG err)
{
	C4DOS.Hf->SetError(this,err);
}

Bool HyperFile::ReadValueHeader(UCHAR *h)
{
	return C4DOS.Hf->ReadValueHeader(this,h);
}

Bool HyperFile::SkipValue(UCHAR h)
{
	return C4DOS.Hf->SkipValue(this,h);
}

Bool HyperFile::WriteChunkStart(LONG id, LONG level)
{
	return C4DOS.Hf->WriteChunkStart(this,id,level);
}

Bool HyperFile::WriteChunkEnd(void)
{
	return C4DOS.Hf->WriteChunkEnd(this);
}

Bool HyperFile::ReadChunkStart(LONG *id, LONG *level)
{
	return C4DOS.Hf->ReadChunkStart(this,id,level);
}

Bool HyperFile::ReadChunkEnd(void)
{
	return C4DOS.Hf->ReadChunkEnd(this);
}

Bool HyperFile::SkipToEndChunk(void)
{
	return C4DOS.Hf->SkipToEndChunk(this);
}

