#ifndef __C4DBASESEQUENCE_H
#define __C4DBASESEQUENCE_H

#include "c4d_baselist.h"
#include "operatingsystem.h"
#include "ge_vector.h"

class BaseObject;
class BaseDocument;
class BaseTrack;
class BaseKey;

#define TIMEMODE_S	0 // path 
#define TIMEMODE_V	1 // velocity 
#define TIMEMODE_A	2 // acceleration

class BaseSequence : public BaseList2D
{
	private:
		BaseSequence(void);
		~BaseSequence(void);
	public:
		BaseTrack			*GetTrack(void) { return (BaseTrack*)C4DOS.Bl->GetMain(this); }
		BaseObject    *GetObject  (void);
		BaseDocument  *GetDocument(void);

		void					SetData(const BaseContainer &bc);
		BaseContainer GetData(void);

		BaseSequence 	*GetClone(LONG flags=0);
		Bool    			CopyTo(BaseSequence *dest, LONG flags=0);
		Bool 					Edit(void); 
		void					SetT1(const BaseTime &time);
		BaseTime			GetT1(void);
		void					SetT2(const BaseTime &time);
		BaseTime			GetT2(void);
		void					SetT3(const BaseTime &time);
		BaseTime			GetT3(void);
		void					SetSoft(Bool b);
		Bool					GetSoft(void);
		void					SetTimeMode(LONG b);
		LONG					GetTimeMode(void);
		void					SetLeftInfluence(Bool b);
		Bool					GetLeftInfluence(void);
		void					SetRightInfluence(Bool b);
		Bool					GetRightInfluence(void);
		void					SetV0(Real r);
		Real					GetV0(void);
		void					SetS0(Real r);
		Real					GetS0(void);

		void					FlushAllKeys(void);
		void					FlushAllTimeKeys(void);
		LONG					GetKeyCount(void);
		LONG					GetTimeKeyCount(void);
		BaseKey				*FindKey(const BaseTime &time);
		BaseTime			GetLeft(void);
		BaseTime			GetRight(void);
		BaseKey				*GetFirstKey(void);
		BaseKey				*GetFirstTimeKey(void);
		Bool					InsertKey(BaseKey *bk);
		Bool					InsertTimeKey(BaseKey *bk);

		BaseSequence *next(void) { return (BaseSequence*)C4DOS.Bl->Next(this); }
		BaseSequence *pred(void) { return (BaseSequence*)C4DOS.Bl->Pred(this); }
};

class FolderSequence : public BaseSequence
{
	private:
		FolderSequence(void);
		~FolderSequence(void);
	public:
		void SetName(const String &s);
		String GetName(void);
		
		BaseObject *GetFirstObject(void);
		void			 InsertObject(BaseObject *op, BaseObject *parent, BaseObject *pred);
};

class SoundSequence : public BaseSequence
{
	private:
		SoundSequence(void);
		~SoundSequence(void);
	public:
		BaseSound *GetSound(void);
		void SetSound(BaseSound *bs);
};

class PluginSequence : public BaseSequence
{
	private:
		PluginSequence(void);
		~PluginSequence(void);
	public:
		LONG GetPluginID(void);
		void *GetPluginData(void);
		void SetPluginData(void *mem);
};

BaseSequence *AllocSequence(LONG type, LONG subtype=0);
#define FreeSequence(v) { if (v) C4DOS.Bl->Free(v); v=NULL;}

#endif
