/////////////////////////////////////////////////////////////
// CINEMA SDK : OBJECTS                                    //
/////////////////////////////////////////////////////////////
// VERSION    : CINEMA 4D V6 AMBER												 //
/////////////////////////////////////////////////////////////
// (c) 1989-2001 MAXON Computer GmbH, all rights reserved	 //
/////////////////////////////////////////////////////////////

#ifndef __C4D_OBJECTPLUGIN_H
#define __C4D_OBJECTPLUGIN_H

#include "ge_math.h"
#include "ge_prepass.h"

class _Filename;
class _String;
class _BaseContainer;
class BaseThread;
class BaseContainer;
class BaseDocument;
class BaseBitmap;
class BaseDraw;
class BaseObject;
class BaseDrawHelp;
class HierarchyHelp;
class PluginObject;
class Particle;
class HyperFile;
class RootTextureString;
class SplineObject;
struct BaseParticle;

#define QUALIFIER_SHIFT	1
#define QUALIFIER_CTRL	2

struct OBJECTPLUGIN
{
	Bool					add_menu;
	BaseBitmap		*icon,*icon_small;
	LONG					disklevel,info;
	Bool					(*MenuPrepare				)(PluginObject *op, BaseDocument *doc);

	Bool					(*AllocSettings			)(PluginObject *op);
	void					(*FreeSettings			)(PluginObject *op);
	Bool					(*CloneSettings			)(PluginObject *src, PluginObject *dst);
	Bool					(*ReadSettings			)(PluginObject *op, HyperFile *hf, LONG level);
	Bool					(*WriteSettings			)(PluginObject *op, HyperFile *hf);
	Bool					(*EditSettings			)(PluginObject *op);

	Bool					(*Message						)(PluginObject *op, LONG msg, void *data);
	Bool					(*ModifyObject			)(PluginObject *mod, BaseDocument *doc, BaseObject *op, const Matrix &op_mg, const Matrix &mod_mg, Real lod, LONG flags, BaseThread *thread);
	void					(*GetDimension			)(PluginObject *op, Vector *mp, Vector *rad); 
	Bool					(*Draw							)(PluginObject *op, BaseDraw *bd, BaseDrawHelp *bh);
	Bool					(*DrawBox						)(PluginObject *op, BaseDraw *bd, BaseDrawHelp *bh);
	Bool					(*DrawHandles				)(PluginObject *op, BaseDraw *bd, BaseDrawHelp *bh);
	BaseObject*		(*GetVirtualObjects )(PluginObject *op, HierarchyHelp *hh);
	void					(*CheckDirty				)(PluginObject *op, BaseDocument *doc);
	LONG					(*DetectHandle      )(PluginObject *op, BaseDraw *bd, LONG x, LONG y, LONG qualifier);
	Bool					(*MoveHandle				)(PluginObject *op, PluginObject *undo, const Matrix &tm, LONG hit_id, LONG qualifier);
	Bool					(*IsAddictedEx			)(PluginObject *op, BaseDocument *doc);
	SplineObject* (*GetContour				)(PluginObject *op, BaseDocument *doc, Real lod, BaseThread *bt);
	Bool					(*GetAllTextures		)(PluginObject *op, RootTextureString *priv);
	void					(*GetData						)(PluginObject *op, _BaseContainer *bc);
	void					(*SetData						)(PluginObject *op, _BaseContainer *bc);
	LONG					(*GetDescription		)(PluginObject *op, LONG id);
	void					(*ModifyParticles   )(PluginObject *op, Particle *pp, BaseParticle *ss, LONG pcnt, Real diff);
};

#endif
