/////////////////////////////////////////////////////////////
// CINEMA SDK : CINEMA OPERATING SYSTEM        						 //
/////////////////////////////////////////////////////////////
// VERSION    : CINEMA 4D V6 AMBER												 //
/////////////////////////////////////////////////////////////
// (c) 1989-2001 MAXON Computer GmbH, all rights reserved	 //
/////////////////////////////////////////////////////////////

// Interface Functions

#ifndef __GEPREPASS_H
#define __GEPREPASS_H

#include "ge_math.h"

// HyperFile::GetError
#define FILEERROR_NONE    	           0 // no error
#define FILEERROR_OPEN							  -1 // problems opening the file
#define FILEERROR_CLOSE								-2 // problems closing the file
#define FILEERROR_READ   							-3 // problems reading the file
#define FILEERROR_WRITE								-4 // problems writing the file
#define FILEERROR_SEEK								-5 // problems seeking the file
#define FILEERROR_INVALID							-6 // invalid parameter or operation (e.g. writing in read-mode)
#define FILEERROR_MEMORY							-7 // not enough memory
#define FILEERROR_WRONG_VALUE       -100 // other value detected than expected
#define FILEERROR_CHUNK_NUMBER      -102 // wrong number of chunks or sub chunks detected
#define FILEERROR_VALUE_NO_CHUNK    -103 // there was a value without any enclosing START/STOP chunks
#define FILEERROR_FILE_END          -104 // the file end was reached without finishing reading
#define FILEERROR_UNKNOWN_VALUE     -105 // unknown value detected

// BaseFile::Open
#define GE_READ		1
#define GE_WRITE	2
#define GE_APPEND	0

#define FILE_NODIALOG		0
#define FILE_DIALOG			1
#define FILE_IGNOREOPEN	2

#define MACTYPE_CINEMA		'C4DC'
#define MACCREATOR_CINEMA	'C4D1'

// BaseFile::Seek
#define GE_START		0
#define GE_RELATIVE	2

// BaseFile::Filer
#define GE_SAVE				1
#define GE_DIRECTORY	2

// BaseBitmap::Init
#define IMAGE_OK					 1
#define IMAGE_NOTEXISTING	-1
#define IMAGE_WRONGTYPE 	-2
#define IMAGE_NOMEM				-3
#define IMAGE_DISKERROR	  -4
#define IMAGE_FILESTRUCT	-5

// BaseBitmap::Save
#define FILTER_TIF								1100
#define FILTER_TGA								1101
#define FILTER_BMP								1102
#define FILTER_IFF								1103
#define FILTER_JPG								1104
#define FILTER_PICT								1105
#define FILTER_PSD								1106
#define FILTER_RLA								1107
#define FILTER_RPF								1108
#define FILTER_B3D								1109
#define FILTER_AVI_SMALL					1120
#define FILTER_AVI_BIG						1121
#define FILTER_AVI_USER						1122
#define FILTER_MOVIE_SMALL				1123
#define FILTER_MOVIE_BIG					1124
#define FILTER_MOVIE_USER					1125
#define FILTER_QTVRSAVER_PANORAMA	1150
#define FILTER_QTVRSAVER_OBJECT		1151

#define AVISAVER_FCCTYPE					10000
#define AVISAVER_FCCHANDLER				10001
#define AVISAVER_LKEY							10002
#define AVISAVER_LDATARATE				10003
#define AVISAVER_LQ								10004

#define QTSAVER_COMPRESSOR				10010
#define QTSAVER_QUALITY						10011
#define QTSAVER_TEMPQUAL					10012
#define QTSAVER_FRAMERATE					10013
#define QTSAVER_KEYFRAMES					10014
#define QTSAVER_PLANES						10015
#define QTSAVER_DATARATE					10016
#define QTSAVER_FRAMEDURATION			10017
#define QTSAVER_MINQUALITY				10018
#define QTSAVER_MINTEMPQUAL				10019
#define QTSAVER_FIXEDFRAMERATE		10020

#define JPGSAVER_QUALITY					10021
#define IMAGESAVER_DPI						10022
#define JPGSAVER_GREYSCALE				10023
#define RLA_OPTIONS								10024

#define RLA_Z											(1<<0)
#define RLA_OBJECTBUFFER					(1<<2)
#define RLA_UV										(1<<3)
#define RLA_NORMAL								(1<<4)
#define RLA_ORIGCOLOR							(1<<5)
#define RLA_COVERAGE							(1<<6)
#define RLA_OBJECTID							(1<<8)
#define RLA_COLOR									(1<<9)
#define RLA_TRANSPARENCY					(1<<10)
#define RLA_SUBPIXEL_WEIGHT				(1<<12)
#define RLA_SUBPIXEL_MASK					(1<<13)

// savebits
#define SAVEBIT_ALPHA             (1<<0)

// GeGetCurrentOS
#define GE_WIN										1
#define GE_MAC										2

// GeGetByteOrder
#define GE_MOTOROLA								1
#define GE_INTEL									2

// GeOutString
#define GEMB_OK										0x0000
#define GEMB_OKCANCEL							0x0001
#define GEMB_ABORTRETRYIGNORE			0x0002
#define GEMB_YESNOCANCEL					0x0003
#define GEMB_YESNO								0x0004
#define GEMB_RETRYCANCEL					0x0005

#define GEMB_ICONSTOP							0x0010
#define GEMB_ICONQUESTION					0x0020
#define GEMB_ICONEXCLAMATION			0x0030
#define GEMB_ICONASTERISK					0x0040

#define GEMB_R_OK									1
#define GEMB_R_CANCEL							2
#define	GEMB_R_ABORT							3
#define GEMB_R_RETRY							4
#define GEMB_R_IGNORE							5
#define GEMB_R_YES								6
#define GEMB_R_NO									7

// result code for mousedrag...
#define MOUSEDRAG_ESCAPE						(1)
#define MOUSEDRAG_FINISHED					(2)
#define MOUSEDRAG_CONTINUE					(3)

// flags for mousedrag...
#define MOUSEDRAG_DONTHIDEMOUSE			(1<<0)				// mousepointer should be visible
#define MOUSEDRAG_NOMOVE						(1<<1)				// mousedrag returns if no mousemove was done
#define MOUSEDRAG_EVERYPACKET				(1<<2)				// receive every packet of the queue, otherwise only data of the last packet

// BaseChannel::InitTexture
#define LOAD_OK            0
#define LOAD_NOMEM			-100
#define LOAD_NOTFOUND   -101
#define LOAD_UNKNOWN    -102
#define LOAD_DOUBLENAME	-105

// hyperfile values
#define HFILE_START   	 1
#define HFILE_STOP 		   2
#define HFILE_CSTOP      3
#define HFILE_CHAR			11
#define HFILE_UCHAR			12
#define HFILE_WORD			13
#define HFILE_UWORD			14
#define HFILE_LONG			15
#define HFILE_ULONG			16
#define HFILE_LLONG			17
#define HFILE_LULONG		18
#define HFILE_REAL			19
#define HFILE_LREAL			20
#define HFILE_BOOL			21
#define HFILE_TIME			22
#define HFILE_VECTOR		23
#define HFILE_LVECTOR		24
#define HFILE_MATRIX		25
#define HFILE_LMATRIX		26

// spline type
#define Tlinear		0
#define Tcubic		1
#define Takima		2
#define Tbspline	3
#define Thermite	4

// spline interpolation
#define Inone			0
#define Inatural	1
#define Iuniform	2
#define Iadaptive	3

// particle bits
#define PARTICLE_VISIBLE	(1<<0)
#define PARTICLE_ALIVE		(1<<1)

// editor modes
#define Mcamera	      0
#define Mobject		    1
#define Maxis		      2
#define Mtexture		  3
#define Mtextureaxis  4
#define Mpoints		    5
#define Mpolygons     7
#define Manimation	  9
#define Mkinematic	 10
#define Mmodel       11
#define Mpaint       12
#define Mdrag      1000

#define RDATA_NAME												5000
#define RDATA_ANTIALIASING								5002
#define RDATA_TRANSPARENCY								5004
#define RDATA_REFLECTION									5005
#define RDATA_SHADOW						 					5006
#define RDATA_RESOLUTION									5007
#define RDATA_XRES												5008
#define RDATA_YRES												5009
#define RDATA_FILMFORMAT									5010
#define RDATA_XFILM												5011
#define RDATA_YFILM												5012
#define RDATA_XPIX												5013
#define RDATA_YPIX												5014
#define RDATA_FRAMESEQUENCE								5016
#define RDATA_FRAMEFROM										5017
#define RDATA_FRAMETO											5018
#define RDATA_FIELD												5019
#define RDATA_FRAMERATE										5020
#define RDATA_ACTIVEOBJECTONLY						5021
#define RDATA_AUTOLIGHT										5022
#define RDATA_LOGFILE											5023
#define RDATA_TEXTUREERROR								5024
#define RDATA_RAYDEPTH										5025
#define RDATA_SHADOWDEPTH									5026
#define RDATA_THRESHOLD										5027
#define RDATA_AASOFTNESS									5028
#define RDATA_FORMAT											5033
#define RDATA_NAMEFORMAT									5035
#define RDATA_ALPHACHANNEL								5036
#define RDATA_STRAIGHTALPHA								5038
#define RDATA_SEPARATEALPHA								5039
#define RDATA_VRGENERATE									5040
#define RDATA_PATH												5041
#define RDATA_HSTEPS											5043
#define RDATA_HSTART											5044
#define RDATA_HEND												5045
#define RDATA_VSTEPS											5046
#define RDATA_VSTART											5047
#define RDATA_VEND												5048
#define RDATA_VOLUMETRICLIGHTING					5052
#define RDATA_LOD													5061
#define RDATA_REFLECTIONDEPTH							5063
#define RDATA_TEXTURES										5064
#define RDATA_VRDEFAULTX									5065
#define RDATA_VRDEFAULTY									5066
#define RDATA_SUPPRESSMP									5067
#define RDATA_ENABLEBLURRY								5069 // BOOL
#define RDATA_AAFILTER										5070 // LONG
#define RDATA_AATHRESHOLD									5071 // REAL
#define RDATA_AAMINLEVEL									5072 // LONG
#define RDATA_AAMAXLEVEL									5073 // LONG
#define RDATA_AAOBJECTPROPERTIES					5074 // BOOL
#define RDATA_TRUECOLORDITHERING					5075 // BOOL
#define RDATA_SAVEIMAGE										5076 // BOOL
#define RDATA_FORMATDEPTH									5077 // LONG
#define	RDATA_AAMIPGLOBAL									5078 // REAL

#define RDATA_GI_ENABLEDIFFUSE						5100 // BOOL
#define RDATA_GI_MAXDIFFUSEDEPTH					5102 // LONG
#define RDATA_GI_MONTECARLOSAMPLES				5104 // LONG
#define RDATA_GI_ACCURACY									5105 // REAL
#define RDATA_GI_MAXRESOLUTION						5106 // LONG
#define RDATA_GI_STRENGTH									5108 // REAL
#define RDATA_CS_ENABLECAUSTICS						5111 // BOOL
#define RDATA_CS_ENABLEVOLUMECAUSTICS			5112 // BOOL
#define RDATA_CS_STEPSIZE									5113 // REAL
#define RDATA_CS_SAMPLERADIUS							5114 // REAL
#define RDATA_CS_ACCURACY									5115 // LONG
#define RDATA_CS_STRENGTH									5116 // REAL
#define RDATA_GI_MINRESOLUTION						5117 // LONG
#define RDATA_GI_RECOMPUTE								5119 // LONG
#define RDATA_GI_SAVESOLUTION							5120 // BOOL
#define RDATA_CS_RECOMPUTE								5121 // LONG
#define RDATA_CS_SAVESOLUTION							5122 // BOOL
#define RDATA_GI_PREPASS									5123 // LONG
#define RDATA_GI_CAMERASOLUTION						5124 // BOOL
#define RDATA_CS_CAMERASOLUTION						5125 // BOOL
#define RDATA_RENDERASEDITOR							5300 // BOOL

#define RDATA_MULTIPASS_SAVEIMAGE					5200 // BOOL
#define RDATA_MULTIPASS_SAVEONEFILE				5201 // BOOL
#define RDATA_MULTIPASS_ENABLE						5202 // BOOL
#define RDATA_MULTIPASS_SAVEFORMAT				5203 // LONG
#define RDATA_MULTIPASS_SAVEOPTIONS				5204 // BaseContainer
#define RDATA_MULTIPASS_LIGHTS						5205 // LONG
#define RDATA_MULTIPASS_FILENAME					5206 // Filename
#define RDATA_MULTIPASS_SAVEDEPTH					5207 // LONG
#define RDATA_MULTIPASS_LIGHTMODE					5208 // LONG
#define RDATA_MULTIPASS_CHANNELS					5209 // BaseContainer
	#define RDATA_MULTIPASS_ACTIVE						 1 // BOOL
	#define RDATA_MULTIPASS_BLENDDATA					 4 // BaseContainer
	#define RDATA_MULTIPASS_SPECIALGROUP			 5 // LONG
#define RDATA_MULTIPASS_SUFFIX						5210 // LONG
#define RDATA_MULTIPASS_SHADOWCORRECTION	5211 // BOOL

#define RDATA_PROJECTFILE									5212 // BOOL

#define RDATA_POSTEFFECTS_ENABLE					5401 // BOOL
#define RDATA_POSTEFFECTS									5402 // BaseContainer
  #define RDATA_POSTEFFECTS_ACTIVE				   1 // BOOL

// scene motion blur
#define RDATA_FX_SMBSUB										1000 // LONG
#define RDATA_FX_SMBSTRENGTH							1001 // REAL
#define RDATA_FX_SMBDITHER								1002 // REAL

// object motion blur
#define RDATA_FX_OMBSTRENGTH							1000 // REAL

// filter strength
#define RDATA_FX_FILTERSTRENGTH						1000 // REAL

// external filter
#define RDATA_FX_PATH											1000 // FILENAME

// comic renderer
#define RDATA_FX_COMICCOLOR								1000 // BOOL
#define RDATA_FX_COMICILLUMINATION				1001 // BOOL
#define RDATA_FX_COMICOUTLINE							1002 // BOOL
#define RDATA_FX_COMICEDGES								1003 // BOOL
#define RDATA_FX_COMICEDGECOLOR						1004 // VECTOR
#define RDATA_FX_COMICBACKCOLOR						1005 // VECTOR
#define RDATA_FX_COMICQUANTIZE						1006 // BOOL
#define RDATA_FX_COMICQUANTIZELEVEL				1007 // LONG

// color correction filter
#define RDATA_FX_CORRECTGAMMA							1000 // REAL
#define RDATA_FX_CORRECTCONTRAST					1001 // REAL
#define RDATA_FX_CORRECTBRIGHTNESS				1002 // REAL

///////////////////bits/////////////////////

// document, material, renderdata, tag, object
#define BIT_ACTIVE					(1<<1) // active element

// objects
#define BIT_AAMAN						(1<<4) // active object in timeline
#define BIT_OFOLD						(1<<6) // folded in object manager
#define BIT_AFOLD						(1<<7) // folded in timeline
#define BIT_CONTROLOBJECT		(1<<9) // control object

#define BIT_TRACKPROCESSED  (1<<16)

///////////////////ID's/////////////////////

// material
#define Mmaterial												5703
#define Mplugin													5705

// objects
#define Opolygon												5100
#define Ospline													5101
#define Olight													5102
#define Ocamera													5103
#define Ofloor													5104
#define Osky														5105
#define Oenvironment										5106
#define Oloft														5107
#define Offd														5108
#define Oparticle												5109
#define Odeflector											5110
#define Ogravitation										5111
#define Orotation												5112
#define Owind														5113
#define Ofriction												5114
#define Oturbulence											5115
#define Oextrude												5116
#define Olathe													5117
#define Osweep													5118
#define Oattractor											5119
#define Obezier													5120
#define Oforeground											5121
#define Obackground											5122
#define Obone														5123
#define Odestructor											5124
#define Ometaball												5125
#define Oinstance												5126
#define Ohypernurbs											5127
#define Obend														5128
#define Obulge													5129
#define Omagnet													5130
#define Oshear													5131
#define Otaper													5133
#define Otwist													5134
#define Owave														5135
#define Ostage													5136
#define Oline														5137
#define Omicrophone											5138
#define Oloudspeaker										5139
#define Onull														5140
#define Oprimitive											5141
#define Osymmetry												5142
#define Owrap														5143
#define Oboole													5144
#define Oexplosion											5145
#define Oformula												5146
#define Omelt														5147
#define Oshatter												5148
#define Owinddeform											5149
#define Oarray													5150
#define Oheadphone											5151
#define Osplineprimitive								5152
#define Oconplane												5153
#define Opluginobject										5154

// tags
#define Tpoint													5600
#define	Tanchor													5608
#define	Tphong													5612
#define	Tdisplay												5613
#define	Tkinematic											5614
#define	Ttexture												5616
#define	Thermite2d											5617
#define	Tprotection											5629
#define	Tparticle												5630
#define	Tmotionblur											5636
#define	Tcompositing										5637
#define	Twww														5647
#define	Tsavetemp												5650
#define Tpolygon												5604
#define	Tuvw														5671
#define	Tsegment												5672
#define	Tpolyselection									5673
#define	Tpointselection									5674
#define Tcoffeeexpression								5675
#define	Ttargetexpression								5676
#define	Tfixexpression									5677
#define	Tsunexpression									5678
#define	Tikexpression										5679
#define Tline														5680
#define Tvertexmap											5682
#define Trestriction										5683
#define	Tmetaball												5684
#define	Tbakeparticle										5685
#define	Tcoffeeplugin  									5686
#define	Texpressionplugin								5687
#define Tsticktexture										5690
#define Tplugin													5691

// keys, sequences, tracks
#define Sfade							5201
#define Spos							5202
#define Sscale						5203
#define Srot							5204
#define Saligntospline		5205
#define Skinematic 				5206
#define Saligntopath		  5207
#define Starget						5208
#define Stexture					5209
#define Smorph						5210
#define Spulsate					5211
#define Svibrate					5212
#define Scoffeeplugin			5213
#define Spla							5214
#define Smarker						5215
#define Sfolder						5216
#define Ssound						5217
#define Stime							5219
#define Sparameter        5220
#define Splugin						5221

#define TEfade						5301
#define TEpos							5302
#define TEscale						5303
#define TErot							5304
#define TEaligntospline		5305
#define TEkinematic 			5306
#define TEaligntopath		  5307
#define TEtarget					5308
#define TEtexture					5309
#define TEmorph						5310
#define TEpulsate					5311
#define TEvibrate					5312
#define TEcoffeeplugin		5313
#define TEpla							5314
#define TEmarker					5315
#define TEfolder					5316
#define TEsound						5317
#define TEtime						5319
#define TEparameter       5320
#define TEplugin					5321

#define SEfade						5401
#define SEpos							5402
#define SEscale						5403
#define SErot							5404
#define SEaligntospline		5405
#define SEkinematic 			5406
#define SEaligntopath		  5407
#define SEtarget					5408
#define SEtexture					5409
#define SEmorph						5410
#define SEpulsate					5411
#define SEvibrate					5412
#define SEcoffeeplugin		5413
#define SEpla							5414
#define SEmarker					5415
#define SEfolder					5416
#define SEsound						5417
#define SEtime						5419
#define SEparameter       5420
#define SEplugin					5421

#define KEfade						5501
#define KEpos							5502
#define KEscale						5503
#define KErot							5504
#define KEaligntospline		5505
#define KEkinematic 			5506
#define KEaligntopath		  5507
#define KEtarget					5508
#define KEtexture					5509
#define KEmorph						5510
#define KEpulsate					5511
#define KEvibrate					5512
#define KEcoffeeplugin		5513
#define KEpla							5514
#define KEmarker					5515
#define KEfolder					5516
#define KEsound						5517
#define KEtime						5519
#define KEparameter       5520
#define KEplugin					5521

// various tag IDs
#define EXPRESSIONTAG_PASS							999103 // LONG

#define PHONGTAG_PHONG									1000 // REAL

#define MOTIONBLURTAG_STRENGTH					1000 // REAL

#define WWWTAG_URL											1000 // STRING
#define WWWTAG_INFO											1001 // STRING

#define COFFEEEXPRESSIONTAG_TEXT				1000 // STRING

#define FIXEXPRESSIONTAG_POINT					1000 // VECTOR

#define TARGETEXPRESSIONTAG_NAME				1000 // STRING

#define IKEXPRESSIONTAG_NAME						1000 // STRING

#define SUNEXPRESSIONTAG_LATITUDE				1000 // REAL
#define SUNEXPRESSIONTAG_DATE						1001 // REAL
#define SUNEXPRESSIONTAG_TIME						1002 // REAL
#define SUNEXPRESSIONTAG_DISTANCE				1003 // REAL

// various key IDs
#define VISIBILITYKEY_VALUE							1000  // REAL

#define ALIGNTOSPLINEKEY_NAME						1000 // STRING
#define ALIGNTOSPLINEKEY_TANGENTIAL			1001 // BOOL

#define KINEMATICKEY_NAME								1000 // STRING

#define TARGETKEY_NAME									1000 // STRING

#define MORPHKEY_NAME										1000 // STRING

#define MARKERKEY_NAME									1000 // STRING

#define ALIGNTOPATH_ANGLE								1000 // REAL
#define ALIGNTOPATH_LOOKAHEAD						1001 // BASETIME

#define TEXTUREKEY_OFFSETX							1000 // REAL
#define TEXTUREKEY_OFFSETY		     			1001 // REAL
#define TEXTUREKEY_LENGTHX	     				1002 // REAL
#define TEXTUREKEY_LENGTHY							1003 // REAL
#define TEXTUREKEY_PROJECTION						1004 // LONG
#define TEXTUREKEY_SIDE			 						1005 // LONG
#define TEXTUREKEY_RESTRICTION					1006 // STRING
#define TEXTUREKEY_TILE			 						1007 // BOOL
#define TEXTUREKEY_SEAMLESS	 						1008 // BOOL
#define TEXTUREKEY_MIX 			 						1009 // BOOL

#define PULSATEKEY_POS_AMP							1000 // VECTOR
#define PULSATEKEY_SCALE_AMP						1001 // VECTOR
#define PULSATEKEY_ROT_AMP							1002 // VECTOR
#define PULSATEKEY_POS_FREQ							1003 // REAL
#define PULSATEKEY_SCALE_FREQ						1004 // REAL
#define PULSATEKEY_ROT_FREQ							1005 // REAL

#define VIBRATEKEY_POS_V								1000 // VECTOR
#define VIBRATEKEY_SCALE_V							1001 // VECTOR
#define VIBRATEKEY_ROT_V								1002 // VECTOR
#define VIBRATEKEY_FREQUENCY						1003 // REAL
#define VIBRATEKEY_SEED									1004 // LONG

#define SOUNDKEY_VOLUME									1000 // REAL
#define SOUNDKEY_BALANCE								1001 // REAL

#define FOLDERKEY_VALUE									1000 // REAL

#define TIMEKEY_P												1000 // VECTOR
#define TIMEKEY_VL											1001 // VECTOR
#define TIMEKEY_VR											1002 // VECTOR

#define PARAMETERKEY_OBJECTID						 600 // LONG

#define PSRKEY_P_X_YY										1003 // REAL
#define PSRKEY_P_Y_YY										1004 // REAL
#define PSRKEY_P_Z_YY										1005 // REAL

#define PSRKEY_VL_X_XX									1006 // BASETIME
#define PSRKEY_VL_Y_XX									1007 // BASETIME
#define PSRKEY_VL_Z_XX									1008 // BASETIME

#define PSRKEY_VL_X_YY									1009 // REAL
#define PSRKEY_VL_Y_YY									1010 // REAL
#define PSRKEY_VL_Z_YY									1011 // REAL

#define PSRKEY_VR_X_XX									1012 // BASETIME
#define PSRKEY_VR_Y_XX									1013 // BASETIME
#define PSRKEY_VR_Z_XX									1014 // BASETIME

#define PSRKEY_VR_X_YY									1015 // REAL
#define PSRKEY_VR_Y_YY									1016 // REAL
#define PSRKEY_VR_Z_YY									1017 // REAL

#define MORPHSEQUENCE_CUBIC							1000 // BOOL
#define TEXTURESEQUENCE_MORPH						1000 // BOOL

// restriction tag
#define RESTRICTION_TAGNAME_01					1100
#define RESTRICTION_TAGNAME_02					1101
#define RESTRICTION_TAGNAME_03					1102
#define RESTRICTION_TAGNAME_04					1103
#define RESTRICTION_TAGNAME_05					1104
#define RESTRICTION_TAGNAME_06					1105

#define RESTRICTION_TAGVAL_01						1200
#define RESTRICTION_TAGVAL_02						1201
#define RESTRICTION_TAGVAL_03						1202
#define RESTRICTION_TAGVAL_04						1203
#define RESTRICTION_TAGVAL_05						1204
#define RESTRICTION_TAGVAL_06						1205

// various object content IDs
#define ATTRACTOROBJECT_STRENGTH				1000 // REAL
#define ATTRACTOROBJECT_SIZE						1001 // VECTOR
#define ATTRACTOROBJECT_SPEEDLIMIT			1002 //	REAL

#define DEFORMOBJECT_SIZE								1000 // VECTOR
#define DEFORMOBJECT_MODE								1001 // LONG
#define DEFORMOBJECT_STRENGTH						1002 // REAL
#define DEFORMOBJECT_ANGLE							1003 // REAL
#define DEFORMOBJECT_CURVATURE					1004 // REAL
#define DEFORMOBJECT_FILLET							1005 // BOOL

#define DEFLECTOROBJECT_SIZEX						1000 // REAL
#define DEFLECTOROBJECT_SIZEY						1001 // REAL
#define DEFLECTOROBJECT_SPLIT						1002 // BOOL
#define DEFLECTOROBJECT_ELASTICITY			1003 // REAL

#define DESTRUCTOROBJECT_RANDOM					1000 // REAL
#define DESTRUCTOROBJECT_SIZE						1001 // VECTOR

#define ENVIRONMENT_AMBIENT							1000 // VECTOR
#define ENVIRONMENT_AMBIENTSTRENGTH			1001 // REAL
#define ENVIRONMENT_FOGENABLE						1002 // BOOL
#define ENVIRONMENT_FOG									1003 // VECTOR
#define ENVIRONMENT_FOGSTRENGTH					1004 // REAL
#define ENVIRONMENT_FOGDISTANCE					1005 // REAL

#define EXPLOSIONOBJECT_PSPEED					1000 // REAL
#define EXPLOSIONOBJECT_ASPEED					1001 // REAL
#define EXPLOSIONOBJECT_ENDSIZE					1002 // REAL
#define EXPLOSIONOBJECT_RANDOM					1003 // REAL
#define EXPLOSIONOBJECT_STRENGTH				1004 // REAL

#define FRICTIONOBJECT_STRENGTH					1000 // REAL
#define FRICTIONOBJECT_SIZE							1001 // VECTOR

#define GRAVITATIONOBJECT_STRENGTH			1000 // REAL
#define GRAVITATIONOBJECT_SIZE					1001 // VECTOR

#define HEADPHONEOBJECT_BASIS						1000 // REAL

#define INSTANCEOBJECT_SOURCE						1000 // STRING

#define LMOBJECT_SHOW										1000 // BOOL
#define LMOBJECT_INNERCONE							1001 // BOOL
#define LMOBJECT_OUTERCONE							1002 // BOOL
#define LMOBJECT_INNERANGLE							1003 // REAL
#define LMOBJECT_OUTERANGLE							1004 // REAL
#define LMOBJECT_FALLOFF								1005 // LONG
#define LMOBJECT_INNERDIST							1006 // REAL
#define LMOBJECT_OUTERDIST							1007 // REAL
#define LMOBJECT_DOPPLER								1008 // BOOL
#define LMOBJECT_DFACTOR								1009 // REAL

#define PARTICLEOBJECT_BIRTHEDITOR			1000 // LONG
#define PARTICLEOBJECT_BIRTHRAYTRACER		1001 // LONG
#define PARTICLEOBJECT_SPEED						1002 // REAL
#define PARTICLEOBJECT_SPEEDVAR					1003 // REAL
#define PARTICLEOBJECT_LIFETIME					1004 // BASETIME
#define PARTICLEOBJECT_LIFETIMEVAR			1005 // REAL
#define PARTICLEOBJECT_SIZE							1006 // REAL
#define PARTICLEOBJECT_SIZEVAR					1007 // REAL
#define PARTICLEOBJECT_ROTATION					1008 // REAL
#define PARTICLEOBJECT_ROTATIONVAR			1009 // REAL
#define PARTICLEOBJECT_TANGENTIAL				1010 // BOOL
#define PARTICLEOBJECT_TYPE							1011 // LONG
#define PARTICLEOBJECT_SIZEX						1012 // REAL
#define PARTICLEOBJECT_SIZEY						1013 // REAL
#define PARTICLEOBJECT_ANGLEH						1014 // REAL
#define PARTICLEOBJECT_ANGLEV						1015 // REAL
#define PARTICLEOBJECT_LASTT            1016 // BASETIME
#define PARTICLEOBJECT_LASTP            1017 // VECTOR
#define PARTICLEOBJECT_START            1018 // BASETIME
#define PARTICLEOBJECT_STOP             1019 // BASETIME
#define PARTICLEOBJECT_SEED             1020 // LONG
#define PARTICLEOBJECT_VISIBILITY       1021 // REAL
#define PARTICLEOBJECT_SHOWOBJECTS			1022 // BOOL

#define CAMERAOBJECT_FOCUS							 500	// REAL
#define CAMERAOBJECT_ZOOM								1000	// REAL
#define CAMERAOBJECT_PROJECTION					1001	// LONG
#define CAMERAOBJECT_DEPTHTYPE					1002  // LONG
#define CAMERAOBJECT_DEPTHFRONT					1003  // REAL
#define CAMERAOBJECT_DEPTHMIDDLE				1004  // REAL
#define CAMERAOBJECT_DEPTHBACK					1005  // REAL
#define CAMERAOBJECT_APERTURE						1006  // REAL
#define CAMERAOBJECT_SHOW								1007  // BOOL

#define ROTATIONOBJECT_STRENGTH					1000 // REAL
#define ROTATIONOBJECT_SIZE							1001 // VECTOR

#define SHATTEROBJECT_ANGLESPEED				1000 // REAL
#define SHATTEROBJECT_SCALE							1001 // REAL
#define SHATTEROBJECT_RANDOM						1002 // REAL
#define SHATTEROBJECT_STRENGTH					1003 // REAL

#define STAGEOBJECT_CNAME								1000 // STRING
#define STAGEOBJECT_SNAME								1001 // STRING
#define STAGEOBJECT_FNAME								1002 // STRING
#define STAGEOBJECT_BNAME								1003 // STRING
#define STAGEOBJECT_ENAME								1004 // STRING

#define TURBULENCEOBJECT_STRENGTH				1000 // REAL
#define TURBULENCEOBJECT_SIZE						1001 // VECTOR

#define WINDOBJECT_STRENGTH							1000 // REAL
#define WINDOBJECT_SIZE									1001 // VECTOR

#define ARRAYOBJECT_RADIUS							1000 // REAL
#define ARRAYOBJECT_COPIES							1001 // LONG
#define ARRAYOBJECT_AMPLITUDE						1002 // REAL
#define ARRAYOBJECT_FREQUENCY						1003 // REAL
#define ARRAYOBJECT_CIRCLEFREQ					1004 // REAL

#define BEZIEROBJECT_SUBRX							1000 // LONG
#define BEZIEROBJECT_SUBRY							1001 // LONG
#define BEZIEROBJECT_SUBX								1002 // LONG
#define BEZIEROBJECT_SUBY								1003 // LONG
#define BEZIEROBJECT_CLOSEX							1004 // BOOL
#define BEZIEROBJECT_CLOSEY							1005 // BOOL

#define BONEOBJECT_FUNCTION							1000 // LONG
#define BONEOBJECT_LIMITRANGE						1001 // BOOL
#define BONEOBJECT_SCALESTRENGTH				1002 // BOOL
#define BONEOBJECT_MINRANGE							1003 // REAL
#define BONEOBJECT_MAXRANGE							1004 // REAL
#define BONEOBJECT_STRENGTH							1005 // REAL
#define BONEOBJECT_LENGTH								1006 // REAL
#define BONEOBJECT_FIXPOSITION					1007 // VECTOR
#define BONEOBJECT_FIXSCALE							1008 // VECTOR
#define BONEOBJECT_FIXROTATION					1009 // VECTOR
#define BONEOBJECT_FIXLENGTH						1010 // REAL

#define BOOLEOBJECT_TYPE								1000 // LONG

#define CONSTRUCTIONPLANE_TYPE					1000	// LONG
#define CONSTRUCTIONPLANE_SPACING				1001	// REAL
#define CONSTRUCTIONPLANE_SUB						1002	// LONG
#define CONSTRUCTIONPLANE_ROUGHSUB			1003	// LONG

#define	EXTRUDEOBJECT_ISOPARM						1000 // LONG
#define EXTRUDEOBJECT_SUB								1001 // LONG
#define EXTRUDEOBJECT_MOVE							1002 // VECTOR
#define EXTRUDEOBJECT_FLIPNORMALS				1003 // BOOL
#define	EXTRUDEOBJECT_HIERARCHIC				1004 // BOOL

#define FFDOBJECT_SIZE									1000 // VECTOR
#define FFDOBJECT_XSUB									1001 // LONG
#define FFDOBJECT_YSUB									1002 // LONG
#define FFDOBJECT_ZSUB									1003 // LONG

#define FORMULAOBJECT_FX								1000 // STRING
#define FORMULAOBJECT_FY								1001 // STRING
#define FORMULAOBJECT_FZ								1002 // STRING
#define FORMULAOBJECT_SIZE							1003 // VECTOR
#define FORMULAOBJECT_DLT								1004 // STRING
#define FORMULAOBJECT_EFFECT						1005 // LONG

#define HYPERNURBSOBJECT_SUBEDITOR			1000 // LONG
#define HYPERNURBSOBJECT_SUBRAY					1001 // LONG

#define	LATHEOBJECT_ISOPARM							1000 // LONG
#define LATHEOBJECT_SUB									1001 // LONG
#define LATHEOBJECT_MOVE								1002 // REAL
#define LATHEOBJECT_SCALE								1003 // REAL
#define LATHEOBJECT_ROTATE							1004 // REAL
#define LATHEOBJECT_FLIPNORMALS					1005 // BOOL

#define	LOFTOBJECT_ISOPARM							1000 // LONG
#define LOFTOBJECT_SUBX									1001 // LONG
#define LOFTOBJECT_SUBY									1002 // LONG
#define LOFTOBJECT_CLOSEY								1003 // BOOL
#define LOFTOBJECT_FLIPNORMALS					1004 // BOOL
#define LOFTOBJECT_LINEAR								1005 // BOOL
#define LOFTOBJECT_ORGANIC							1006 // BOOL
#define LOFTOBJECT_ADAPTIVEY						1007 // BOOL
#define LOFTOBJECT_FITUV								1008 // BOOL

#define MELTOBJECT_RADIUS								1000 // REAL
#define MELTOBJECT_VERTRANDOM						1001 // REAL
#define MELTOBJECT_RADRANDOM						1002 // REAL
#define	MELTOBJECT_SIZE									1003 // REAL
#define MELTOBJECT_NOISESCALE						1004 // REAL
#define MELTOBJECT_STRENGTH							1005 // REAL

#define METABALLOBJECT_THRESHOLD				1000 // REAL
#define METABALLOBJECT_SUBEDITOR				1001 // REAL
#define METABALLOBJECT_SUBRAY						1002 // REAL
#define METABALLOBJECT_EXPONENTIAL			1003 // BOOL

#define SPLINEOBJECT_TYPE								1000 // LONG
#define SPLINEOBJECT_INTERPOLATION			1001 // LONG
#define SPLINEOBJECT_CLOSED							1002 // BOOL
#define SPLINEOBJECT_SUB								1003 // LONG
#define SPLINEOBJECT_ANGLE							1004 // REAL

#define	SWEEPOBJECT_ISOPARM							1000 // LONG
#define SWEEPOBJECT_GROWTH							1002 // REAL
#define	SWEEPOBJECT_SCALE								1003 // REAL
#define SWEEPOBJECT_ROTATE							1004 // REAL
#define SWEEPOBJECT_PARALLEL						1005 // BOOL
#define SWEEPOBJECT_RAILDIRECTION				1006 // BOOL
#define SWEEPOBJECT_RAILSCALE						1007 // BOOL
#define SWEEPOBJECT_CONSTANT						1008 // BOOL
#define SWEEPOBJECT_BANKING							1009 // BOOL
#define SWEEPOBJECT_FLIPNORMALS					1010 // BOOL
#define SWEEPOBJECT_KEEPSEGMENTS				1011 // BOOL
#define SWEEPOBJECT_BIRAIL							1012 // BOOL

#define SYMMETRYOBJECT_PLANE						1000 // LONG
#define SYMMETRYOBJECT_WELD							1001 // BOOL
#define SYMMETRYOBJECT_TOLERANCE				1002 // REAL
#define SYMMETRYOBJECT_ONPLANE					1003 // BOOL

#define WINDDEFORMOBJECT_AMPLITUDE			1000 // REAL
#define WINDDEFORMOBJECT_FREQUENCY			1001 // REAL
#define WINDDEFORMOBJECT_TURBULENCE			1002 // REAL
#define WINDDEFORMOBJECT_FX							1003 // REAL
#define WINDDEFORMOBJECT_FY							1004 // REAL
#define WINDDEFORMOBJECT_FLAG						1005 // LONG
#define WINDDEFORMOBJECT_SIZE						1006 // REAL

#define WRAPOBJECT_XSANGLE							1000 // REAL
#define WRAPOBJECT_XEANGLE							1001 // REAL
#define WRAPOBJECT_YSANGLE							1002 // REAL
#define WRAPOBJECT_YEANGLE							1003 // REAL
#define WRAPOBJECT_RADIUS								1004 // REAL
#define WRAPOBJECT_MOVE									1005 // REAL
#define WRAPOBJECT_TYPE									1006 // LONG
#define WRAPOBJECT_WIDTH								1007 // REAL
#define WRAPOBJECT_HEIGHT								1008 // REAL
#define WRAPOBJECT_ZSCALE								1009 // REAL
#define WRAPOBJECT_TENSION							1010 // REAL

// special handling: light object
#define LIGHT_INC	100

#define MAXGLOW_CNT 			5
#define MAXSTREAK_CNT 		5
#define MAXLENSFLARE_CNT 40

#define LIGHT_LENS_POS								 10000 // REAL
#define LIGHT_LENS_SIZE								 10001 // REAL
#define LIGHT_LENS_COL							   10002 // VECTOR
#define LIGHT_LENS_TYP							   10003 // LONG

#define LIGHT_STREAK_RANDOM						 20000 // BOOL
#define LIGHT_STREAK_RANDOMLEN				 20001 // BOOL
#define LIGHT_STREAK_STERN						 20002 // BOOL
#define LIGHT_STREAK_ANZ							 20003 // LONG
#define LIGHT_STREAK_HOLEANZ					 20004 // LONG
#define LIGHT_STREAK_SIZE							 20005 // REAL
#define LIGHT_STREAK_SV								 20006 // REAL
#define LIGHT_STREAK_HOLEWIDTH  			 20007 // REAL
#define LIGHT_STREAK_DICKE		  			 20008 // REAL
#define LIGHT_STREAK_ROT			  			 20009 // REAL
#define LIGHT_STREAK_COL							 20010 // VECTOR
#define LIGHT_STREAK_TYP							 20011 // LONG

#define LIGHT_GLOW_SIZE								 30000 // REAL
#define LIGHT_GLOW_SV									 30001 // REAL
#define LIGHT_GLOW_COL								 30002 // VECTOR
#define LIGHT_GLOW_TYP								 30003 // LONG

#define LIGHT_HALO_SIZE								 40000 // REAL
#define LIGHT_HALO_SV									 40001 // REAL
#define LIGHT_HALO_COL								 40002 // VECTOR
#define LIGHT_HALO_TYP								 40003 // LONG

#define LIGHT_SHADOW_BIAS							 50000 // REAL
#define LIGHT_SHADOW_DENSITY					 50001 // REAL
#define LIGHT_SHADOW_PARRADIUS				 50002 // REAL
#define LIGHT_SHADOW_CONEANGLE				 50003 // REAL
#define LIGHT_SHADOW_AREARADIUS				 50004 // REAL
#define LIGHT_SHADOW_MAPSIZEX					 50005 // LONG
#define LIGHT_SHADOW_MAPSIZEY					 50006 // LONG
#define LIGHT_SHADOW_SMPRADIUS				 50007 // LONG
#define LIGHT_SHADOW_TRANSPARENCY			 50008 // BOOL
#define LIGHT_SHADOW_ABSOLUTE					 50009 // BOOL
#define LIGHT_SHADOW_OUTLINE					 50010 // BOOL
#define LIGHT_SHADOW_CONE							 50011 // BOOL
#define LIGHT_SHADOW_SOFTCONE					 50012 // BOOL
#define LIGHT_SHADOW_COLOR						 50013 // VECTOR
#define LIGHT_SHADOW_AREASAMPLES			 50014 // REAL
#define LIGHT_SHADOW_CLIPINFLUENCE		 50015 // BOOL

#define LIGHT_VNSTRUCT_NOISE					 60000 // LONG
#define LIGHT_VNSTRUCT_OCTAVES				 60001 // LONG
#define LIGHT_VNSTRUCT_VELOCITY				 60002 // REAL
#define LIGHT_VNSTRUCT_RSCALE					 60003 // REAL
#define LIGHT_VNSTRUCT_AMPLITUDE   		 60004 // REAL
#define LIGHT_VNSTRUCT_WINDVELOCITY		 60005 // REAL
#define LIGHT_VNSTRUCT_OFFSET					 60006 // REAL
#define LIGHT_VNSTRUCT_WIND						 60007 // VECTOR
#define LIGHT_VNSTRUCT_LOCAL					 60008 // BOOL
#define LIGHT_VNSTRUCT_VSCALE					 60009 // VECTOR

#define LIGHT_VLSTRUCT_CUSTOMCOL			 70000 // BOOL
#define LIGHT_VLSTRUCT_FALLOFF				 70001 // BOOL
#define LIGHT_VLSTRUCT_EDGEFALLOFF		 70002 // BOOL
#define LIGHT_VLSTRUCT_COLFALLOFF			 70003 // BOOL
#define LIGHT_VLSTRUCT_STRENGTH				 70004 // REAL
#define LIGHT_VLSTRUCT_EDGESTRENGTH		 70005 // REAL
#define LIGHT_VLSTRUCT_JITTER					 70006 // REAL
#define LIGHT_VLSTRUCT_ICOL						 70007 // VECTOR
#define LIGHT_VLSTRUCT_OCOL						 70008 // VECTOR
#define LIGHT_VLSTRUCT_INNERDIST			 70009 // REAL
#define LIGHT_VLSTRUCT_OUTERDIST 			 70010 // VECTOR
#define LIGHT_VLSTRUCT_OUTERDISTY			 70011 // BOOL
#define LIGHT_VLSTRUCT_OUTERDISTZ			 70012 // BOOL
#define LIGHT_VLSTRUCT_BRIGHTNESS			 70019 // REAL
#define LIGHT_VLSTRUCT_SAMPLEDIST			 70020 // REAL
#define LIGHT_VLSTRUCT_DUST						 70021 // REAL
#define LIGHT_VLSTRUCT_ADDITIVE				 70022 // BOOL
#define LIGHT_VLSTRUCT_ANGLESCALE			 70023 // BOOL

#define LIGHT_LFSTRUCT_GLOWSV					 80000 // REAL
#define LIGHT_LFSTRUCT_REFLSV					 80001 // REAL
#define LIGHT_LFSTRUCT_GLOWHELL				 80002 // REAL
#define LIGHT_LFSTRUCT_REFLHELL				 80003 // REAL
#define LIGHT_LFSTRUCT_SCALE					 80004 // REAL
#define LIGHT_LFSTRUCT_ROTATION				 80005 // REAL
#define LIGHT_LFSTRUCT_REFERENCE			 80006 // REAL
#define LIGHT_LFSTRUCT_LIGHTPARAM			 80007 // BOOL
#define LIGHT_LFSTRUCT_FADEBEHIND			 80008 // BOOL
#define LIGHT_LFSTRUCT_FADENEAR				 80009 // BOOL
#define LIGHT_LFSTRUCT_FADEOFFSCREEN	 80010 // BOOL
#define LIGHT_LFSTRUCT_SCALEGLOW			 80011 // BOOL
#define LIGHT_LFSTRUCT_SCALEREFLEXES	 80012 // BOOL
#define LIGHT_LFSTRUCT_LENSANZ				 80013 // LONG
#define LIGHT_LFSTRUCT_LENSGLOW				 80014 // LONG
#define LIGHT_LFSTRUCT_LENSREFL				 80015 // LONG

#define LIGHT_MAIN_F									 90000 // VECTOR
#define LIGHT_MAIN_HELL								 90001 // REAL
#define LIGHT_MAIN_TYPE								 90002 // LONG
#define LIGHT_MAIN_STYPE							 90003 // LONG
#define LIGHT_MAIN_VTYPE							 90004 // LONG
#define LIGHT_MAIN_NTYPE 							 90005 // LONG
#define LIGHT_MAIN_SHOWLI							 90006 // BOOL
#define LIGHT_MAIN_SHOWVL							 90007 // BOOL
#define LIGHT_MAIN_SHOWCLIP						 90008 // BOOL
#define LIGHT_MAIN_DARK								 90009 // BOOL
#define LIGHT_MAIN_INNERANGLE					 90010 // REAL
#define LIGHT_MAIN_OUTERANGLE					 90011 // REAL
#define LIGHT_MAIN_INNERRADIUS				 90012 // REAL
#define LIGHT_MAIN_OUTERRADIUS				 90013 // REAL
#define LIGHT_MAIN_FALLOFF						 90014 // LONG
#define LIGHT_MAIN_AMBIENT						 90015 // BOOL
#define LIGHT_MAIN_NODIFFUSE					 90016 // BOOL
#define LIGHT_MAIN_NOSPECULAR					 90017 // BOOL
#define LIGHT_MAIN_INNERCONE					 90018 // BOOL
#define LIGHT_MAIN_INNERCOLOR					 90019 // BOOL
#define LIGHT_MAIN_COLFALLOFF					 90020 // BOOL
#define LIGHT_MAIN_CONTRAST						 90021 // REAL
#define LIGHT_MAIN_ASPECTRATIO				 90022 // REAL
#define LIGHT_MAIN_BRIGHTNESS					 90023 // REAL
#define LIGHT_MAIN_INNERDIST					 90024 // REAL
#define LIGHT_MAIN_OUTERDIST					 90025 // REAL
#define LIGHT_MAIN_ICOL								 90026 // VECTOR
#define LIGHT_MAIN_NEARCLIP						 90027 // BOOL
#define LIGHT_MAIN_FARCLIP						 90028 // BOOL
#define LIGHT_MAIN_NEARFROM						 90029 // REAL
#define LIGHT_MAIN_NEARTO							 90030 // REAL
#define LIGHT_MAIN_FARFROM						 90031 // REAL
#define LIGHT_MAIN_FARTO							 90032 // REAL
#define LIGHT_MAIN_SEPARATEPASS        90033 // BOOL

#define LIGHT_CAUSTIC_ENABLE					 91000 // BOOL
#define LIGHT_CAUSTIC_ENERGY				   91001 // REAL
#define LIGHT_CAUSTIC_PHOTONS				   91002 // LONG
#define LIGHT_VOLCAUSTIC_ENABLE		     91003 // BOOL
#define LIGHT_VOLCAUSTIC_ENERGY			   91004 // REAL
#define LIGHT_VOLCAUSTIC_PHOTONS       91005 // LONG
#define LIGHT_VOLCAUSTIC_FALLOFF		   91006 // LONG
#define LIGHT_VOLCAUSTIC_INNERRADIUS   91007 // REAL
#define LIGHT_VOLCAUSTIC_OUTERRADIUS   91008 // REAL

// primitive object
#define PRIMITIVE_CUBE									1000
#define PRIMITIVE_SPHERE								1001
#define PRIMITIVE_PLATONIC							1002
#define PRIMITIVE_CONE									1003
#define PRIMITIVE_TORUS									1004
#define PRIMITIVE_DISC									1005
#define PRIMITIVE_TUBE									1006
#define PRIMITIVE_FIGURE								1007
#define PRIMITIVE_PYRAMID								1008
#define PRIMITIVE_PLANE									1009
#define PRIMITIVE_FRACTAL								1010
#define PRIMITIVE_CYLINDER							1011 // based upon cone
#define PRIMITIVE_CAPSULE								1012 // based upon cone
#define PRIMITIVE_OILTANK								1013 // based upon cone
#define PRIMITIVE_RELIEF								1014
#define PRIMITIVE_POLY									1015

// spline primitive object
#define PRIMITIVE_FORMULA								1016
#define PRIMITIVE_TEXT									1017
#define PRIMITIVE_NSIDE									1018
#define PRIMITIVE_4SIDE									1019
#define PRIMITIVE_CIRCLE								2000
#define PRIMITIVE_ARC										2001
#define PRIMITIVE_CISSOID								2002
#define PRIMITIVE_CYCLOID								2003
#define PRIMITIVE_PROFILE								2004
#define PRIMITIVE_HELIX									2005
#define PRIMITIVE_RECTANGLE							2006
#define PRIMITIVE_FLOWER								2007
#define PRIMITIVE_STAR									2008
#define PRIMITIVE_COGWHEEL							2009
#define PRIMITIVE_CONTOUR								2010

// primitive parameter
#define PRIM_CUBE_LEN										1100 // VECTOR   - Side Length [>=0.0]
#define PRIM_CUBE_SEP										1101 // BOOL	   - Seperate Surfaces [only evaluated if no fillet is specified]
#define PRIM_CUBE_SUBX									1102 // LONG     - X-Subdivision [>0]
#define PRIM_CUBE_SUBY									1103 // LONG     - Y-Subdivision [>0]
#define PRIM_CUBE_SUBZ									1104 // LONG     - Z-Subdivision [>0]
#define PRIM_CUBE_FILLET								1105 // REAL     - Fillet Radius [>=0.0; <=len.x/2; <=len.y/2; <=len.z/2]
#define PRIM_CUBE_SUBF									1106 // LONG     - Fillet Subdivision [0:No Fillet]

#define PRIM_SPHERE_RAD									1110 // REAL     - Radius [>=0.0]
#define PRIM_SPHERE_SUB									1111 // LONG     - Subdivision [>0]
#define PRIM_SPHERE_TYPE								1112 // LONG     - Type [0:Standard; 1:Tetra; 2:Hexa; 3:Octa; 4:Icosa]
#define PRIM_SPHERE_PERFECT							1113 // BOOL		 - Keep Perfect Shape During Rendering

#define PRIM_PLATONIC_RAD								1120 // REAL	   - Outer Radius	[>=0.0]
#define PRIM_PLATONIC_SUB								1121 // LONG     - Subdivision [>=0]
#define PRIM_PLATONIC_TYPE							1122 // LONG     - Type [0:Tetra; 1:Hexa; 2:Octa; 3:Dodeca; 4:Icosa; 5:Bucky]

#define PRIM_CONE_TRAD									1130 // REAL 	   - Ptop Radius [>=0.0]
#define PRIM_CONE_BRAD									1131 // REAL  	 - Pbottom Radius [>=0.0]
#define PRIM_CONE_HEIGHT								1132 // REAL     - Height [>=0.0]
#define PRIM_CONE_HSUB									1133 // LONG  	 - Height Segments [>0]
#define PRIM_CONE_CSUB									1134 // LONG     - Cap Segments [>0]
#define PRIM_CONE_SEG										1135 // LONG     - Rotational Segments [>2]
#define PRIM_CONE_TYPE									1136 // LONG	   - Type [0:No Caps; 1:Caps; 2:Ptop Bevel; 3:Pbottom Bevel; 4:Both Bevels]
#define PRIM_CONE_TFILLET								1137 // REAL	   - Ptop Fillet [>=0.0, <=TopRad, TopFil+BotFil<=Height]
#define PRIM_CONE_BFILLET								1138 // REAL	   - Botom Fillet [>=0.0, <=TopRad, TopFil+BotFil<=Height]
#define PRIM_CONE_TFILLETY							1139 // REAL     - Ptop Fillet Y [>=0.0, overwrites Y component of TFILLET]
#define PRIM_CONE_BFILLETY							1140 // REAL     - Pbottom Fillet Y [>=0.0, overwrites Y component of BFILLET]
#define PRIM_CONE_FSUB									1141 // LONG	   - Fillet Subdivision [>0]
#define PRIM_CONE_TFILANGLE							1142 // REAL	   - Ptop Fillet Angle [>=0.0, <=pi/2]
#define PRIM_CONE_BFILANGLE							1143 // REAL	   - Ptop Fillet Angle [>=0.0, <=pi/2]

#define PRIM_TORUS_OUTERRAD							1150 // REAL	   - Outer Radius [>=0.0]
#define PRIM_TORUS_INNERRAD							1151 // REAL     - Inner Radius [>=0.0,<=Outer Radius]
#define PRIM_TORUS_CSUB									1152 // LONG	   - Cross Section Segments [>2]
#define PRIM_TORUS_SEG									1153 // LONG     - Rotational Segments [>2]

#define PRIM_DISC_IRAD									1160 // REAL	   - Inner Radius [>=0.0]
#define PRIM_DISC_ORAD									1161 // REAL	   - Outer Radius [>=Inner Radius]
#define PRIM_DISC_CSUB									1162 // LONG	   - Disc-Cap Subdivision [>0]
#define PRIM_DISC_SEG										1163 // LONG     - Rotational Segments [>2]

#define PRIM_TUBE_ORAD									1170 // REAL	   - Outer Radius [>=0.0]
#define PRIM_TUBE_IRAD									1171 // REAL     - Inner Radius [>=0.0,<=Outer Radius]
#define PRIM_TUBE_HEIGHT								1172 // REAL     - Height [>=0.0]
#define PRIM_TUBE_HSUB									1174 // LONG     - Height Subdivision [>0]
#define PRIM_TUBE_CSUB									1175 // LONG     - Cap Subdivision [>0]
#define PRIM_TUBE_SEG										1176 // LONG     - Rotational Segments [>2]

#define PRIM_FIGURE_HEIGHT							1210 // REAL     - Height [>=0.0]
#define PRIM_FIGURE_SUB									1211 // LONG     - Subdivision [>2]

#define PRIM_PYRAMID_LEN								1220 // VECTOR   - Pyramid Size [>=0.0]
#define PRIM_PYRAMID_SUB								1221 // LONG     - Segments [>0]

#define PRIM_PLANE_WIDTH								1230 // REAL     - Plane Width [>=0.0]
#define PRIM_PLANE_HEIGHT								1231 // REAL     - Plane Height [>=0.0]
#define PRIM_PLANE_SUBW									1232 // LONG     - Width Segments [>0]
#define PRIM_PLANE_SUBH									1233 // LONG     - Height Segments [>0]

#define PRIM_FRACTAL_LEN								1240 // REAL     - Fractal Size [>=0.0]
#define PRIM_FRACTAL_SUBW								1241 // LONG     - Width Segments [>0]
#define PRIM_FRACTAL_SUBH								1242 // LONG     - Height Segments [>0]
#define PRIM_FRACTAL_ROUGH							1243 // REAL     - Rough Furrows [>=0.0; <=1.0]
#define PRIM_FRACTAL_FINE								1244 // REAL     - Fine Furrows [>=0.0; <=1.0]
#define PRIM_FRACTAL_SCALE							1245 // REAL     - Scale [>=0.1; <=1.0]
#define PRIM_FRACTAL_BLEVEL							1246 // REAL     - Pbottom Level (Sea) [>=0.0; <=1.0]
#define PRIM_FRACTAL_TLEVEL							1247 // REAL     - Ptop Level (Plateau) [>=0.0; <=1.0]
#define PRIM_FRACTAL_MULTIFRACTAL				1248 // BOOL     - Multifractal Function
#define PRIM_FRACTAL_BORDERS						1249 // BOOL     - Borders At Sealevel
#define PRIM_FRACTAL_SPHERICAL					1250 // BOOL     - Generate Spherical Shape

#define PRIM_RELIEF_LEN									1260 // REAL     - Relief Size [>=0.0]
#define PRIM_RELIEF_SUBW								1261 // LONG     - Width Segments [>0]
#define PRIM_RELIEF_SUBH								1262 // LONG     - Height Segments [>0]
#define PRIM_RELIEF_TEXTURE							1263 // STRING	 - Texture Name
#define PRIM_RELIEF_SPHERICAL						1264 // BOOL		 - Generate Spherical Shape
#define PRIM_RELIEF_BLEVEL							1265 // REAL     - Pbottom Level (Sea) [>=0.0; <=1.0]
#define PRIM_RELIEF_TLEVEL							1266 // REAL     - Ptop Level (Plateau) [>=0.0; <=1.0]

#define PRIM_POLY_WIDTH									1270 // REAL	   - Width [>=0.0]
#define PRIM_POLY_HEIGHT								1271 // REAL	   - Height [>=Inner Radius]
#define PRIM_POLY_SUB										1272 // LONG	   - Subdivision [>0]
#define PRIM_POLY_TRIANG								1273 // BOOL     - Tri Or Quad

// valid for several primitives
#define PRIM_SLICE											1200 // BOOL     - Slicing
#define PRIM_SLICEFROM									1201 // REAL     - Start Angle
#define PRIM_SLICETO										1202 // REAL     - End Angle [>=Start Angle]
#define PRIM_REGULAR										1203 // BOOL		 - Regular Grid
#define PRIM_REGULARWIDTH								1204 // REAL		 - Regular Grid Width

#define PRIM_AXIS												1300 // LONG	   - Direction [0:+X; 1:-X; 2:+Y; 3:-Y; 4:+Z; 5:-Z]


// spline primitive parameter
#define PRIM_ARC_TYPE										2000 // LONG		 - Arc Type [0:Arc; 1:Sector; 2:Segment; 3:Ring]
#define PRIM_ARC_RADIUS									2001 // REAL		 - Radius [>=0.0]
#define PRIM_ARC_INNER									2002 // REAL		 - Inner Radius (Ring) [>=0.0]
#define PRIM_ARC_START									2005 // REAL		 - Start Angle
#define PRIM_ARC_END										2006 // REAL		 - End Angle

#define PRIM_CIRCLE_ELLIPSE							2010 // BOOL		 - Ellipse Shape
#define PRIM_CIRCLE_RING								2011 // BOOL		 - Ring Shape
#define PRIM_CIRCLE_RADIUS							2012 // REAL		 - Radius [>=0.0]
#define PRIM_CIRCLE_RADIUSY							2013 // REAL		 - Y Radius [>=0.0]
#define PRIM_CIRCLE_INNER								2014 // REAL		 - Inner Radius (Ring) [>=0.0]

#define PRIM_CISSOID_TYPE								2020 // LONG		 - Type [0:Cissoid; 1:Lemniscate; 2:Strophoid]
#define PRIM_CISSOID_TENSION						2021 // REAL		 - Value Range
#define PRIM_CISSOID_WIDTH							2023 // REAL		 - Width
#define PRIM_CISSOID_SUB								2024 // LONG		 - Subdivision [>=3]

#define PRIM_HELIX_START								2031 // REAL		 - Start Angle
#define PRIM_HELIX_END									2032 // REAL		 - End Angle
#define PRIM_HELIX_RADIUS1							2033 // REAL		 - First Radius [>=0.0]
#define PRIM_HELIX_RADIUS2							2034 // REAL		 - Second Radius [>=0.0]
#define PRIM_HELIX_HEIGHT								2035 // REAL		 - Height
#define PRIM_HELIX_RADIALBIAS						2036 // REAL		 - Radial Bias [>=0.0; <=1.0]
#define PRIM_HELIX_HEIGHTBIAS						2037 // REAL		 - Height Bias [>=0.0; <=1.0]
#define PRIM_HELIX_SUB									2038 // LONG		 - Subdivision [>=3]

#define PRIM_CYCLOID_TYPE								2040 // LONG		 - Type [0:Cycloid; 1:Epicycloid; 2:Hypocycloid]
#define PRIM_CYCLOID_RADIUS							2041 // REAL		 - Radius [>=0.0]
#define PRIM_CYCLOID_R									2042 // REAL		 - R [>=0.0]
#define PRIM_CYCLOID_A									2043 // REAL		 - A [>=0.0]
#define PRIM_CYCLOID_START							2044 // REAL		 - Start Angle
#define PRIM_CYCLOID_END								2045 // REAL		 - End Angle

#define PRIM_PROFILE_TYPE								2050 // LONG		 - Type [0:H; 1:L; 2:T; 3:U; 4:Z]
#define PRIM_PROFILE_A									2051 // REAL		 - A [>=0.0]
#define PRIM_PROFILE_B									2052 // REAL		 - B [>=0.0]
#define PRIM_PROFILE_S									2053 // REAL		 - S [>=0.0]
#define PRIM_PROFILE_T    							2054 // REAL		 - T [>=0.0]

#define PRIM_RECTANGLE_WIDTH						2060 // REAL		 - Width [>=0.0]
#define PRIM_RECTANGLE_HEIGHT						2061 // REAL		 - Height[>=0.0]
#define PRIM_RECTANGLE_ROUNDING					2062 // BOOL		 - Rounding
#define PRIM_RECTANGLE_RADIUS						2063 // REAL		 - Rounding Radius [>=0.0]

#define PRIM_STAR_IRAD									2070 // REAL		 - Inner Radius [>=0.0]
#define PRIM_STAR_ORAD									2071 // REAL		 - Outer Radius [>=0.0]
#define PRIM_STAR_TWIST									2072 // REAL		 - Twist [>=-1.0; <=1.0]
#define PRIM_STAR_POINTS								2073 // LONG		 - Points [>=3]

#define PRIM_COGWHEEL_TEETH							2080 // LONG		 - Number Of Teeth [>=5]
#define PRIM_COGWHEEL_IRAD							2082 // REAL		 - Inner Radius [>=0.0]
#define PRIM_COGWHEEL_MRAD							2083 // REAL		 - Middle Radius [>=0.0]
#define PRIM_COGWHEEL_ORAD							2084 // REAL		 - Outer Radius [>=0.0]
#define PRIM_COGWHEEL_BEVEL							2085 // REAL		 - Bevel [>=0.0; <=1.0]

#define PRIM_FLOWER_IRAD								2090 // REAL		 - Inner Radius [>=0.0]
#define PRIM_FLOWER_ORAD								2091 // REAL		 - Outer Radius [>=0.0]
#define PRIM_FLOWER_PETALS							2092 // LONG		 - Number Of Petals [>=3]

#define PRIM_FORMULA_X									2100 // STRING	 - X component
#define PRIM_FORMULA_Y									2101 // STRING	 - Y component
#define PRIM_FORMULA_Z									2102 // STRING	 - Z component
#define PRIM_FORMULA_TMIN								2103 // REAL		 - Start Value
#define PRIM_FORMULA_TMAX								2104 // REAL		 - End Value
#define PRIM_FORMULA_SAMPLES						2105 // LONG		 - Samples [>=2]
#define PRIM_FORMULA_CUBIC							2106 // BOOL		 - Cubic Extrapolation

#define PRIM_TEXT_USE_PS								2110 // BOOL		 - Specify If PSFont was chosen
#define PRIM_TEXT_TEXT									2111 // STRING	 - Text
#define PRIM_TEXT_ALIGN									2112 // LONG		 - [0:Left; 1:Middle; 2:Right]
#define PRIM_TEXT_HSPACING							2113 // REAL		 - Horizontal Spacing
#define PRIM_TEXT_VSPACING							2114 // REAL		 - Vertical Spacing
#define PRIM_TEXT_HEIGHT								2115 // REAL		 - Vertical Spacing
#define PRIM_TEXT_SEPARATE							2116 // BOOL		 - separate letters (make editable)

#define PRIM_4SIDE_TYPE									2120 // LONG		 - Type [0:Diamond; 1:Kite; 2:Parallelogram; 3:Trapezium]
#define PRIM_4SIDE_A										2121 // REAL		 - a [>=0.0]
#define PRIM_4SIDE_B										2122 // REAL		 - b [>=0.0]
#define PRIM_4SIDE_ANGLE								2123 // REAL		 - Angle [>-Rad(90.0); <Rad(90.0)]

#define PRIM_NSIDE_RADIUS								2130 // REAL		 - Radius [>=0.0]
#define PRIM_NSIDE_SIDES								2131 // LONG		 - Sides [>=3]
#define PRIM_NSIDE_ROUNDING							2132 // BOOL		 - Rounding
#define PRIM_NSIDE_RRADIUS							2133 // REAL		 - Rounding Radius[>=0.0]

#define PRIM_CONTOUR_TEXTURE						2140 // STRING	 - Filename
#define PRIM_CONTOUR_WIDTH							2141 // REAL		 - Width [>=0.0]
#define PRIM_CONTOUR_TOLERANCE					2142 // REAL		 - Tolerance [>0.0]

#define PRIM_PLANE											2300 // LONG		 - Direction [0:XY; 1:ZY; 2:XZ]
#define PRIM_REVERSE										2301 // BOOL		 - Reverse Spline Order

// caps for Extrude/Sweep/Lathe/Loft NURBS
#define CAP_START								2000	// LONG [0:No Cap; 1:Cap; 2:Rounding; 3:Rounding And Cap]
#define CAP_END									2001	// LONG [0:No Cap; 1:Cap; 2:Rounding; 3:Rounding And Cap]
#define CAP_ROUNDING						2002	// LONG [0:Linear; 1:Convex; 2:Concave; 3:Half Circle; 4:1 Step; 5:2 Steps; 6:Engraved]
#define CAP_STARTSTEPS					2003	// LONG
#define CAP_ENDSTEPS						2004  // LONG
#define CAP_STARTRADIUS					2005  // REAL
#define CAP_ENDRADIUS						2006  // REAL
#define CAP_HULLINWARD					2007  // BOOL
#define CAP_HOLEINWARD					2008  // BOOL
#define CAP_CONSTRAIN						2009  // BOOL
#define CAP_REGULAR							2010	// BOOL
#define CAP_REGULARWIDTH				2011	// REAL
#define CAP_PHONG								2012	// REAL

// projections
#define Pperspective  0
#define Pparallel			1
#define Pleft				  2
#define Pright			  3
#define Pfront			  4
#define Pback			    5
#define Ptop				  6
#define Pbottom				7
#define Pmilitary		  8
#define Pfrog			    9
#define Pbird				 10
#define Pgentleman	 11
#define Pisometric	 12
#define Pdimetric    13

// modeling commands
#define MCOMMAND_SPLINE_HARDINTERPOLATION			100
#define MCOMMAND_SPLINE_SOFTINTERPOLATION			101
#define MCOMMAND_SPLINE_REORDER								102
#define MCOMMAND_SPLINE_REVERSE								103
#define MCOMMAND_SPLINE_MOVEDOWN							104
#define MCOMMAND_SPLINE_MOVEUP								105
#define MCOMMAND_SPLINE_CHAMFER								106
#define MCOMMAND_SPLINE_ROUND									107
#define MCOMMAND_SPLINE_JOINSEGMENT						109
#define MCOMMAND_SPLINE_BREAKSEGMENT					110
#define MCOMMAND_SPLINE_EQUALLENGTH						111
#define MCOMMAND_SPLINE_EQUALDIRECTION				112
#define MCOMMAND_SPLINE_LINEUP								113
#define MCOMMAND_SPLINE_CREATEOUTLINE					114
#define MCOMMAND_SPLINE_PROJECT								115
#define MCOMMAND_SPLINE_ADDPOINT							116
#define MCOMMAND_SELECTALL										200
#define MCOMMAND_DESELECTALL									201
#define MCOMMAND_SELECTINVERSE								202
#define MCOMMAND_SELECTCONNECTED							203
#define MCOMMAND_SELECTGROW										204
#define MCOMMAND_SELECTSHRINK									205
#define MCOMMAND_SELECTPOINTTOPOLY						206
#define MCOMMAND_SELECTPOLYTOPOINT						207
#define MCOMMAND_SELECTADJACENT								208
#define MCOMMAND_GENERATESELECTION						209
#define MCOMMAND_HIDESELECTED									211
#define MCOMMAND_HIDEUNSELECTED								212
#define MCOMMAND_HIDEINVERT										213
#define MCOMMAND_UNHIDE												214
#define MCOMMAND_QUANTIZE											216
#define MCOMMAND_REVERSENORMALS								217
#define MCOMMAND_ALIGNNORMALS									218
#define MCOMMAND_SETVALUE											219
#define MCOMMAND_SPLIT												220
#define MCOMMAND_TRIANGULATE									221
#define MCOMMAND_UNTRIANGULATE								222
#define MCOMMAND_DELETE												224
#define MCOMMAND_CRUMPLE											225
#define MCOMMAND_WELD													226
#define MCOMMAND_OPTIMIZE											227
#define MCOMMAND_DISCONNECT										228
#define MCOMMAND_CREATEPOLYGON								229
#define MCOMMAND_MAKEEDITABLE									230
#define MCOMMAND_SMOOTHSHIFT									231
#define MCOMMAND_EXTRUDE											232
#define MCOMMAND_EXTRUDEINNER									233
#define MCOMMAND_BEVEL												234
#define MCOMMAND_ARRAY												235
#define MCOMMAND_CLONE												236
#define MCOMMAND_MIRROR												237
#define MCOMMAND_MATRIXEXTRUDE								238
#define MCOMMAND_NORMALMOVE										239
#define MCOMMAND_NORMALSCALE									240
#define MCOMMAND_NORMALROTATE									241
#define MCOMMAND_SUBDIVIDE										242
#define MCOMMAND_EXPLODESEGMENTS							243
#define MCOMMAND_KNIFE												244
#define MCOMMAND_CURRENTSTATETOOBJECT					245
#define MCOMMAND_JOIN													246

// tool settings for selection commands
#define	MDATA_QUANTIZE_ENABLEX								2003 // BOOL
#define	MDATA_QUANTIZE_ENABLEY								2004 // BOOL
#define	MDATA_QUANTIZE_ENABLEZ								2005 // BOOL
#define	MDATA_QUANTIZE_VAL										2006 // VECTOR
#define	MDATA_SETVALUE_SETX										2008 // BOOL
#define	MDATA_SETVALUE_SETY										2009 // BOOL
#define	MDATA_SETVALUE_SETZ										2010 // BOOL
#define	MDATA_SETVALUE_VAL										2011 // VECTOR
#define	MDATA_SETVALUE_SYSTEM									2012 // LONG
#define MDATA_SPLINE_ROUNDPOINTS							2015 // LONG
#define MDATA_SPLINE_ROUNDTYPE								2016 // LONG
#define MDATA_SPLINE_CHAMFERFLAT							2017 // BOOL
#define MDATA_SPLINE_CHAMFERRADIUS						2018 // REAL
#define MDATA_SPLINE_FREEHANDTOLERANCE				2020 // REAL
#define MDATA_SPLINE_OUTLINE									2021 // REAL
#define MDATA_SPLINE_PROJECTMODE							2022 // LONG
#define MDATA_SPLINE_ADDPOINTSEGMENT					2023 // LONG
#define MDATA_SPLINE_ADDPOINTPOSITION					2024 // REAL
#define MDATA_CRUMPLE_MODE										2025 // LONG
#define MDATA_CRUMPLE_INANDOUT								2026 // BOOL
#define MDATA_CRUMPLE_VAL											2027 // VECTOR
#define MDATA_DISCONNECT_PRESERVEGROUPS				2028 // BOOL
#define	MDATA_SMOOTHSHIFT_ANGLE								2030 // REAL
#define	MDATA_SMOOTHSHIFT_OFFSET							2031 // REAL
#define MDATA_EXTRUDE_PRESERVEGROUPS					2032 //	BOOL
#define MDATA_EXTRUDE_OFFSET									2033 // REAL
#define	MDATA_EXTRUDE_ANGLE										2034 // REAL
#define	MDATA_EXTRUDE_VARIANCE								2035 // REAL
#define MDATA_EXTRUDEINNER_PRESERVEGROUPS			2036 //	BOOL
#define MDATA_EXTRUDEINNER_OFFSET							2037 //	REAL
#define	MDATA_EXTRUDEINNER_ANGLE							2038 // REAL
#define	MDATA_EXTRUDEINNER_VARIANCE						2039 // REAL
#define MDATA_BEVEL_PRESERVEGROUPS						2040 //	BOOL
#define MDATA_BEVEL_OFFSET1										2041 // REAL
#define MDATA_BEVEL_OFFSET2										2042 // REAL
#define	MDATA_BEVEL_ANGLE											2043 // REAL
#define	MDATA_BEVEL_VARIANCE1									2044 // REAL
#define	MDATA_BEVEL_VARIANCE2									2045 // REAL
#define	MDATA_ARRAY_XCNT											2046 // LONG
#define	MDATA_ARRAY_YCNT											2047 // LONG
#define	MDATA_ARRAY_ZCNT											2048 // LONG
#define	MDATA_ARRAY_HOLES											2049 // REAL
#define	MDATA_ARRAY_OFFSET										2050 // VECTOR
#define MDATA_ARRAY_LOCALMOVE									2051 // VECTOR
#define MDATA_ARRAY_LOCALSCALE								2052 // VECTOR
#define MDATA_ARRAY_LOCALROTATE								2053 // VECTOR
#define MDATA_ARRAY_KEEPFORM									2054 // BOOL
#define	MDATA_CLONE_CNT												2055 // LONG
#define	MDATA_CLONE_HOLES											2056 // REAL
#define	MDATA_CLONE_AXIS											2057 // LONG
#define	MDATA_CLONE_OFFSET										2058 // REAL
#define	MDATA_CLONE_SCALE											2059 // REAL
#define	MDATA_CLONE_ROTATION									2060 // REAL
#define MDATA_CLONE_LOCALMOVE									2062 // VECTOR
#define MDATA_CLONE_LOCALSCALE								2063 // VECTOR
#define MDATA_CLONE_LOCALROTATE								2064 // VECTOR
#define MDATA_CLONE_KEEPFORM									2065 // BOOL
#define MDATA_BRIDGE_TRIANGULATE							2066 // BOOL
#define MDATA_MIRROR_SNAPPOINTS								2069 // BOOL
#define MDATA_MIRROR_DUPLICATE								2070 // BOOL
#define MDATA_MIRROR_WELD											2071 // BOOL
#define MDATA_MIRROR_TOLERANCE								2072 // REAL
#define MDATA_MIRROR_SYSTEM										2067 // LONG
#define MDATA_MIRROR_PLANE										2068 // LONG
#define MDATA_MIRROR_VALUE										2073 // REAL
#define MDATA_MIRROR_POINT										2074 // VECTOR
#define MDATA_MIRROR_VECTOR										2075 // VECTOR
#define MDATA_OPTIMIZE_TOLERANCE							2076 // REAL
#define MDATA_OPTIMIZE_POINTS									2077 // BOOL
#define MDATA_OPTIMIZE_POLYGONS								2078 // BOOL
#define MDATA_OPTIMIZE_UNUSEDPOINTS						2079 // BOOL
#define	MDATA_SPLINE_OUTLINESEPARATE					2080 // BOOL
#define MDATA_CREATEPOLYGON_KEEPQUADRANGLE		2081 // BOOL
#define MDATA_CROSSSECTION_ANGLE							2082 // REAL
#define MDATA_MATRIXEXTRUDE_STEPS							2083 // LONG
#define MDATA_MATRIXEXTRUDE_MOVE							2084 // VECTOR
#define MDATA_MATRIXEXTRUDE_SCALE							2085 // VECTOR
#define MDATA_MATRIXEXTRUDE_ROTATE						2086 // VECTOR
#define MDATA_MATRIXEXTRUDE_POLYGONCOORDS			2087 // BOOL
#define MDATA_MATRIXEXTRUDE_MOVEFROM					2088 // REAL
#define MDATA_MATRIXEXTRUDE_MOVETO						2089 // REAL
#define MDATA_MATRIXEXTRUDE_SCALEFROM					2090 // REAL
#define MDATA_MATRIXEXTRUDE_SCALETO						2091 // REAL
#define MDATA_MATRIXEXTRUDE_ROTATEFROM				2092 // REAL
#define MDATA_MATRIXEXTRUDE_ROTATETO					2093 // REAL
#define MDATA_MATRIXEXTRUDE_VARIANCE					2094 // LONG
#define MDATA_NORMALMOVE_VALUE								2095 // REAL
#define MDATA_NORMALSCALE_VALUE								2096 // REAL
#define MDATA_NORMALROTATE_VALUE							2097 // REAL
#define MDATA_SUBDIVIDE_HYPER									2098 // BOOL
#define MDATA_SUBDIVIDE_ANGLE									2099 // REAL
#define MDATA_SUBDIVIDE_SPLINESUB							2100 // LONG
#define MDATA_SUBDIVIDE_SUB										2101 // LONG
#define MDATA_SELECTFREE_TOLERANCE						2102 // BOOL
#define MDATA_SELECTFREE_VISIBLE							2103 // BOOL
#define MDATA_SELECTPOLY_TOLERANCE						2104 // BOOL
#define MDATA_SELECTPOLY_VISIBLE							2105 // BOOL
#define MDATA_SELECTRECT_TOLERANCE						2106 // BOOL
#define MDATA_SELECTRECT_VISIBLE							2107 // BOOL
#define MDATA_SELECTLIVE_VISIBLE							2108 // BOOL
#define MDATA_SELECTLIVE_RADIUS								2109 // LONG
#define MDATA_KNIFE_P1												2110 // VECTOR
#define MDATA_KNIFE_V1												2111 // VECTOR
#define MDATA_KNIFE_P2												2112 // VECTOR
#define MDATA_KNIFE_V2												2113 // VECTOR
#define MDATA_KNIFE_RESTRICT									2114 // BOOL
#define MDATA_KNIFE_ANGLE											2115 // REAL
#define MDATA_MAGNET_NEARESTPOINT							2116 // BOOL
#define MDATA_MAGNET_RADIUS										2117 // REAL
#define MDATA_MAGNET_WIDTH										2118 // REAL
#define MDATA_MAGNET_TYPE											2119 // LONG
#define MDATA_MIRROR_SELECTIONS								2120 // BOOL
#define MDATA_UNTRIANGULATE_ANGLE							2121 // BOOL
#define MDATA_MIRROR_ONPLANE									2122 // BOOL
#define MDATA_VERTEXPAINT_ENABLE							2123 // BOOL
#define MDATA_VERTEXPAINT_MODE								2124 // LONG
#define MDATA_VERTEXPAINT_STRENGTH						2125 // REAL
#define MDATA_SETVERTEX_VALUE									4000 // REAL
#define MDATA_SETVERTEX_MODE									4001 // LONG

#define SNAP_ENABLE														3002 // BOOL
#define SNAP_POINT														3003 // BOOL
#define SNAP_EDGE															3004 // BOOL
#define SNAP_POLYGON													3005 // BOOL
#define SNAP_GRID															3006 // BOOL
#define SNAP_GRIDLINE													3007 // BOOL
#define SNAP_SPLINE														3008 // BOOL
#define SNAP_SPLINEGRID												3009 // BOOL
#define SNAP_AXIS															3010 // BOOL
#define SNAP_PLANE														3011 // BOOL
#define SNAP_TANGENT													3012 // BOOL
#define SNAP_PERPENDICULAR										3013 // BOOL
#define SNAP_FACECENTER												3014 // BOOL
#define SNAP_MIDPOINT													3015 // BOOL
#define SNAP_SPLINESUB												3016 // LONG
#define SNAP_RADIUS														3017 // LONG
#define SNAP_METHOD														3018 // LONG
#define SNAP_PLANE_SPACING										3019 // REAL
#define SNAP_PLANE_SUB												3020 // LONG
#define SNAP_PLANE_ROUGHSUB										3021 // LONG
#define SNAP_PLANE_ENABLE											3022 // LONG
#define SNAP_PLANE_DYNAMICGRID								3023 // LONG
#define SNAP_LOCAL_MOVEENABLE									3024 // BOOL
#define SNAP_LOCAL_MOVE												3025 // REAL
#define SNAP_LOCAL_SCALEENABLE								3026 // BOOL
#define SNAP_LOCAL_SCALE											3027 // REAL
#define SNAP_LOCAL_ROTATEENABLE								3028 // BOOL
#define SNAP_LOCAL_ROTATE											3029 // REAL
#define SNAP_LOCAL_TEXTUREENABLE							3030 // BOOL
#define SNAP_LOCAL_TEXTURE										3031 // REAL
#define SNAP_LOCAL_TOOLENABLE									3032 // BOOL
#define SNAP_LOCAL_TOOL												3033 // REAL

// send modeling command
#define MODIFY_ALL              0
#define MODIFY_POINTSELECTION   1
#define MODIFY_POLYGONSELECTION 2

// world preferences
#define WPREF_UNITS_BASIC										 10000 // LONG
#define WPREF_UNITS_TIME										 10001 // LONG
#define WPREF_UNITS_COLOR										 10002 // LONG
#define WPREF_UNITS_USEUNITS								 10003 // BOOL
#define WPREF_REFRESHTIME										 10004 // LONG
#define WPREF_RATIO_X												 10005 // REAL
#define WPREF_RATIO_Y												 10006 // REAL
#define WPREF_BACKGROUND_COL								 10007 // VECTOR
#define WPREF_FILMFORMAT_COL								 10008 // VECTOR
#define WPREF_HORIZON_COL										 10009 // VECTOR
#define WPREF_GRID_MAJOR_COL								 10010 // VECTOR
#define WPREF_GRID_MINOR_COL								 10011 // VECTOR
#define WPREF_SPLINESTART_COL								 10012 // VECTOR
#define WPREF_SPLINEEND_COL									 10013 // VECTOR
#define WPREF_CAMERA_COL										 10014 // VECTOR
#define WPREF_PARTICLE_COL									 10015 // VECTOR
#define WPREF_PMODIFIER_COL									 10016 // VECTOR
#define WPREF_BONE_COL											 10017 // VECTOR
#define WPREF_MODIFIER_COL									 10018 // VECTOR
#define WPREF_ACTIVEPOINT_COL								 10019 // VECTOR
#define WPREF_INACTIVEPOINT_COL							 10020 // VECTOR
#define WPREF_TANGENT_COL										 10021 // VECTOR
#define WPREF_ACTIVEPOLYGON_COL							 10022 // VECTOR
#define WPREF_INACTIVEPOLYGON_COL						 10023 // VECTOR
#define WPREF_TEXTURE_COL										 10024 // VECTOR
#define WPREF_TEXTUREAXIS_COL								 10025 // VECTOR
#define WPREF_ACTIVEBOX_COL									 10026 // VECTOR
#define WPREF_ANIMPATH_COL									 10027 // VECTOR
#define WPREF_XAXIS_COL											 10028 // VECTOR
#define WPREF_YAXIS_COL											 10029 // VECTOR
#define WPREF_ZAXIS_COL											 10030 // VECTOR
#define WPREF_WXAXIS_COL										 10031 // VECTOR
#define WPREF_WYAXIS_COL										 10032 // VECTOR
#define WPREF_WZAXIS_COL										 10033 // VECTOR
#define WPREF_SELECT_AXIS_COL								 10034 // VECTOR
#define WPREF_LAYER0_COL										 10035 // VECTOR
#define WPREF_LAYER1_COL										 10036 // VECTOR
#define WPREF_LAYER2_COL										 10037 // VECTOR
#define WPREF_LAYER3_COL										 10038 // VECTOR
#define WPREF_LAYER4_COL										 10039 // VECTOR
#define WPREF_LAYER5_COL										 10040 // VECTOR
#define WPREF_LAYER6_COL										 10041 // VECTOR
#define WPREF_LAYER7_COL										 10042 // VECTOR
#define WPREF_VERTEXSTART_COL								 10043 // VECTOR
#define WPREF_VERTEXEND_COL									 10044 // VECTOR
#define WPREF_HPB_SYSTEM											1001 // BOOL
#define WPREF_CENTER													1002 // BOOL
#define WPREF_BACKUP													1003 // BOOL
#define WPREF_RENDERMATS											1004 // BOOL
#define WPREF_TABLETT													1005 // BOOL
#define WPREF_SCALE_AXES											1006 // BOOL
#define WPREF_SEMI_TRANSPARENCY								1007 // BOOL
#define WPREF_OPENGL													1008 // BOOL
#define WPREF_LINK_SELECTION									1009 // BOOL
#define WPREF_ONLY_ACTIVEVIEW									1010 // BOOL
#define WPREF_LOAD_MANAGER										1011 // BOOL
#define WPREF_SAVE_OPENGL											1012 // BOOL
#define WPREF_SAVE_PARTICLES									1013 // BOOL
#define WPREF_SAVE_LAYOUT											1014 // BOOL
#define WPREF_RENDER_SAFE											1015 // BOOL
#define WPREF_ACTION_SAFE											1016 // BOOL
#define WPREF_TITLE_SAFE											1017 // BOOL
#define WPREF_MAX_UNDOS												1018 // LONG
#define WPREF_MAX_LAST												1019 // LONG
#define WPREF_ACTION_FAC											1020 // REAL
#define WPREF_TITLE_FAC												1021 // REAL
#define WPREF_SEMI_FAC												1022 // REAL
#define WPREF_OPENGL_AA_LINES									1023 // BOOL
#define WPREF_OPENGL_PERSPECT									1024 // BOOL
#define WPREF_OPENGL_MIPMAP										1025 // BOOL
#define WPREF_USE_TEXTURES										1026 // BOOL
#define WPREF_OPENGL_FASTREDRAW								1027 // BOOL
#define WPREF_OPENGL_FASTLIVESELECTION				1028 // BOOL
#define WPREF_USE_QUICKTIME										1029 // BOOL
#define WPREF_TABLET_HIRES										1030 // BOOL
#define WPREF_BUBBLEHELP1										 21001
#define WPREF_BUBBLEHELP2										 21002
#define WPREF_THREADPRIORITY								 21003
#define WPREF_MENUICON											 21004
#define WPREF_MENUSHORTCUT									 21005
#define WPREF_INACTIVEBORDER								 21006
#define WPREF_ACTIVEBORDER									 21007
#define WPREF_DIALOGSTYLE										 21008
#define WPREF_DIALOGALIGN										 21009
#define WPREF_DELAY_MENU										 21010
#define WPREF_DELAY_MENUSCROLL							 21011
#define WPREF_DELAY_CMDPOPUP								 21012
#define WPREF_DELAY_DRAGSCROLL							 21013
#define WPREF_DELAY_SCROLLER1								 21014
#define WPREF_DELAY_SCROLLER2								 21015
#define WPREF_DELAY_TABSWITCH								 21016
#define WPREF_COLOR_EDGEWH									 21020
#define WPREF_COLOR_EDGELT									 21021
#define WPREF_COLOR_EDGEDK									 21022
#define WPREF_COLOR_EDGEBL									 21023
#define WPREF_COLOR_TIMESLIDER							 21024
#define WPREF_COLOR_BG											 21025
#define WPREF_COLOR_BGEDIT									 21026
#define WPREF_COLOR_BGGADGET								 21027
#define WPREF_COLOR_BGFOCUS									 21028
#define WPREF_COLOR_TEXT										 21029
#define WPREF_COLOR_TEXTFOCUS								 21030
#define WPREF_COLOR_DBARFG1									 21031
#define WPREF_COLOR_DBARBG1									 21032
#define WPREF_COLOR_DBARFG2									 21033
#define WPREF_COLOR_DBARBG2									 21034
#define WPREF_COLOR_BGSELTEXT								 21035
#define WPREF_COLOR_FGSELTEXT								 21036
#define WPREF_PATTERN_TIMESLIDER						 21040
#define WPREF_PATTERN_BG										 21041
#define WPREF_PATTERN_BGEDIT								 21042
#define WPREF_PATTERN_BGGADGET							 21043
#define WPREF_PATTERN_BGFOCUS								 21044
#define WPREF_PATTERN_DBARBG1								 21045
#define WPREF_PATTERN_DBARBG2								 21046
#define WPREF_FONT_STANDARD									 21050
#define WPREF_FONT_MONO											 21051
#define WPREF_PATTERN_USE										 21060
#define WPREF_NAVSERVICE										 21061

// mouse cursors
#define MOUSE_HIDE								 0
#define MOUSE_SHOW							 	 1
#define MOUSE_NORMAL							 2
#define MOUSE_BUSY								 3
#define MOUSE_CROSS								 4
#define MOUSE_QUESTION						 5
#define MOUSE_ZOOM_IN							 6
#define MOUSE_ZOOM_OUT						 7
#define MOUSE_FORBIDDEN						 8
#define MOUSE_DELETE							 9
#define MOUSE_COPY								10
#define MOUSE_INSERTCOPY					11
#define MOUSE_INSERTCOPYDOWN			12
#define MOUSE_MOVE								13
#define MOUSE_INSERTMOVE					14
#define MOUSE_INSERTMOVEDOWN			15
#define MOUSE_ARROW_H							16
#define MOUSE_ARROW_V							17
#define MOUSE_ARROW_HV						18
#define MOUSE_POINT_HAND					19
#define MOUSE_MOVE_HAND						20
#define MOUSE_IBEAM								21

// global messages
#define MSG_NEWDOCUMENT					 605
#define MSG_DOCUMENTCHANGED			 604
#define MSG_NEWACTIVEOBJECT			 603
#define MSG_ACTIVEOBJECTCHANGED	 602
#define MSG_REDRAWALL						 601
#define MSG_TIMELINECHANGED		 -1001 
#define MSG_BROWSERCHANGED		 -1002 
#define MSG_MATCHANGED         -1004
#define MSG_ACTIVETAGCHANGED	 -1007 
#define MSG_TOOLCHANGED					4294141029//0xFFF36465,

// global message flags
#define MSG_PARAM_ANIMATE          2
#define MSG_PARAM_NOEXPRESSIONS	   8

// draw flags
#define DA_NO_THREAD						(1<<1)
#define DA_NO_REDUCTION					(1<<2)
#define DA_ONLY_ACTIVE					(1<<3)
#define DA_ONLY_BACK						(1<<4)
#define DA_ONEPLANE							(1<<6)
#define DA_NO_KEYFRAME					(1<<7)
#define DA_NO_ANIMATION					(1<<8)
#define DA_ONLY_ACTIVE_VIEW			(1<<10)
#define DA_NO_EXPRESSIONS				(1<<12)
#define DA_FORCECALCULATION			(1<<13)

// animate scene/object flags
#define ANIMATE_NO_PARTICLES		(1<<2)
#define ANIMATE_QUICK						(1<<3)
#define ANIMATE_NO_KEYFRAME			(1<<4)
#define ANIMATE_ONLY_DOWNWARDS	(1<<5)

// undo types
#define UNDO_OBJECT_REC				 			1
#define UNDO_OBJECT						 			2
#define UNDO_OBJECT_DATA_REC   			3
#define UNDO_OBJECT_DATA			 			4
#define UNDO_MATERIAL_DATA     			5
#define UNDO_TAG_DATA					 			6
#define UNDO_KEY_DATA					 			7
#define UNDO_SEQUENCE_DATA     			8
#define UNDO_TRACK_DATA				 			9
#define UNDO_OBJECT_NEW						 10
#define UNDO_TAG_NEW							 11
#define UNDO_KEY_NEW							 12
#define UNDO_SEQUENCE_NEW     		 13
#define UNDO_TRACK_NEW						 14
#define UNDO_MATERIAL_NEW					 15
#define UNDO_OBJECT_DEL						 17
#define UNDO_TAG_DEL							 18
#define UNDO_KEY_DEL							 19
#define UNDO_SEQUENCE_DEL     		 20
#define UNDO_TRACK_DEL						 21
#define UNDO_MATERIAL_DEL     		 22
#define UNDO_OBJECT_ACTIVE    		 23

// handle types
#define HANDLE_MINI				0
#define HANDLE_SMALL			1
#define HANDLE_MIDDLE			2
#define HANDLE_BIG				3

// colors
#define COLOR_BACKGROUND			 0
#define COLOR_FILMFORMAT			 1
#define COLOR_HORIZON					 2
#define COLOR_GRID_MAJOR			 3
#define COLOR_GRID_MINOR			 4
#define COLOR_SPLINESTART			 5
#define COLOR_SPLINEEND				 6
#define COLOR_CAMERA					 7
#define COLOR_PARTICLE				 8
#define COLOR_PMODIFIER				 9
#define COLOR_BONE						10
#define COLOR_MODIFIER				11
#define COLOR_ACTIVEPOINT			12
#define COLOR_INACTIVEPOINT		13
#define COLOR_TANGENT					14
#define COLOR_ACTIVEPOLYGON		15
#define COLOR_INACTIVEPOLYGON 16
#define COLOR_TEXTURE					17
#define COLOR_TEXTUREAXIS			18
#define COLOR_ACTIVEBOX				19
#define COLOR_ANIMPATH				20
#define COLOR_XAXIS						21
#define COLOR_YAXIS						22
#define COLOR_ZAXIS						23
#define COLOR_WXAXIS					24
#define COLOR_WYAXIS					25
#define COLOR_WZAXIS					26
#define COLOR_SELECT_AXIS			27
#define COLOR_LAYER0					28
#define COLOR_LAYER1					29
#define COLOR_LAYER2					30
#define COLOR_LAYER3					31
#define COLOR_LAYER4					32
#define COLOR_LAYER5					33
#define COLOR_LAYER6					34
#define COLOR_LAYER7					35
#define COLOR_VERTEXSTART			36
#define COLOR_VERTEXEND				37

// dirty flags
#define DIRTY_MATRIX						(1<<1)
#define DIRTY_DATA							(1<<2)

// vflags
#define VFLAG_INTERNALRENDERER	(1<<1)
#define VFLAG_EXTERNALRENDERER	(1<<2)
#define VFLAG_ISOPARM						(1<<3)
#define VFLAG_POLYGONAL				  (1<<4)

// hierarchy clone flags
#define HCLONE_ASIS							(1<<0)
#define HCLONE_ASPOLY						(1<<1)
#define HCLONE_ASLINE						(1<<2)
#define HCLONE_ASSPLINE					(1<<3)

// error string dialog
#define IS_NOTHING			0
#define IS_DEGREE				1
#define IS_PERCENT			2
#define IS_METER				3
#define IS_LONG					5

#define Greater					0
#define GreaterOrEqual	1
#define Less						2
#define LessOrEqual			3
#define Between					4
#define BetweenOrEqual	5
#define BetweenOrEqualX	6
#define BetweenOrEqualY	7
#define Different				8

// parameter tracks
#define PARAM_SOLID		0
#define PARAM_TOGGLE	1
#define PARAM_LINEAR	2
#define PARAM_SCALE		3

// material data
#define MAT_DIFFUSION_AFFECT_SPECULAR     1000 // BOOL
#define MAT_DIFFUSION_AFFECT_REFLECTION   1001 // BOOL
#define MAT_DIFFUSION_AFFECT_LUMINANCE    1002 // BOOL
#define MAT_TRANSPARENCY_FRESNEL          1003 // BOOL
#define MAT_TRANSPARENCY_ADDITIVE			    1004 // BOOL
#define MAT_TRANSPARENCY_N                1005 // REAL
#define MAT_ENVIRONMENT_TILESX            1006 // REAL
#define MAT_ENVIRONMENT_TILESY 						1007 // REAL
#define MAT_FOG_DISTANCE                  1008 // REAL
#define MAT_BUMP_STRENGTH                 1009 // REAL
#define MAT_BUMP_MIPFALLOFF               1010 // BOOL
#define MAT_ALPHA_DELTA                   1011 // VECTOR
#define MAT_ALPHA_SOFT                    1012 // BOOL
#define MAT_ALPHA_IMAGEALPHA              1013 // BOOL
#define MAT_ALPHA_INVERT                  1014 // BOOL
#define MAT_SPECULAR_WIDTH                1016 // REAL
#define MAT_SPECULAR_HEIGHT               1017 // REAL
#define MAT_SPECULAR_MODE                 1018 // LONG
#define MAT_GLOW_INNER                    1019 // REAL
#define MAT_GLOW_OUTER                    1020 // REAL
#define MAT_GLOW_RADIUS                   1021 // REAL
#define MAT_GLOW_RANDOM                   1022 // REAL
#define MAT_GLOW_FREQUENCY                1023 // REAL
#define MAT_GLOW_USEMATERIAL              1024 // BOOL
#define MAT_DISPLACEMENT_STRENGTH         1025 // REAL
#define MAT_DISPLACEMENT_HEIGHT           1026 // REAL
#define MAT_ALPHA_PMALPHA									1027 // BOOL
#define MAT_GI_RECEIVE										1100 // BOOL
#define MAT_GI_GENERATE										1101 // BOOL
#define MAT_GI_RECEIVESTRENGTH						1102 // REAL
#define MAT_CS_RECEIVE										1108 // BOOL
#define MAT_CS_GENERATE										1109 // BOOL
#define MAT_CS_SAMPLERADIUS								1110 // REAL
#define MAT_CS_ACCURACY										1111 // LONG
#define MAT_GI_GENERATESTRENGTH						1112 // REAL
#define MAT_CS_GENERATESTRENGTH						1113 // REAL
#define MAT_CS_RECEIVESTRENGTH						1114 // REAL
#define MAT_GI_SATURATION									1115 // REAL

#define MAT_TRANSPARENCY_DISPERSION				1120 // REAL
#define MAT_TRANSPARENCY_ACCURACY					1121 // REAL
#define MAT_TRANSPARENCY_MINSAMPLES				1122 // LONG
#define MAT_TRANSPARENCY_MAXSAMPLES				1123 // LONG

#define MAT_REFLECTION_DISPERSION					1130 // REAL
#define MAT_REFLECTION_ACCURACY						1131 // REAL
#define MAT_REFLECTION_MINSAMPLES					1132 // LONG
#define MAT_REFLECTION_MAXSAMPLES					1133 // LONG

#define MAT_SHADINGMODEL									1134 // LONG
#define MAT_SHADINGMODEL_ROUGHNESS				1135 // REAL
#define MAT_DIFFUSE_FALLOFF								1136 // REAL
#define MAT_SPECULAR_FALLOFF							1137 // REAL
#define MAT_SHADINGMODEL_INTENSITY				1138 // REAL
#define MAT_SPECULAR_INNERWIDTH						1139 // REAL

// paintmesh bits
#define PAINTMESH_QUAD   (1<<1) // polygon is quadrangle
#define PAINTMESH_SEL    (1<<6) // polygon selected

// im-/export formats
#define FORMAT_PREF				1000
#define FORMAT_C4D_4			1001
#define FORMAT_C4D_5			1002
#define FORMAT_C4D_6			1003
#define FORMAT_LWS				1004
#define FORMAT_LWO				1005
#define FORMAT_DXF				1006
#define FORMAT_TDD				1007
#define FORMAT_Q3D				1008
#define FORMAT_ILL				1009
#define FORMAT_DEM				1010
#define FORMAT_VRML1			1011
#define FORMAT_3DS				1012
#define FORMAT_VRML2			1013
#define FORMAT_WAVE				1014
#define FORMAT_D3D				1015
#define FORMAT_CAT				1016
#define FORMAT_FACT				1017
#define FORMAT_WAV				1018
#define FORMAT_FCV				1019
#define FORMAT_L4D				1020
#define FORMAT_C4D_6_XML	1021
#define FORMAT_P4D				1022

// registry
#define RE_WINDOW						   1
#define RE_OBJECT						   3
#define RE_TRACK						   4
#define	RE_SEQUENCE					   5
#define RE_KEY							   6
#define RE_TAG							   7
#define RE_MATERIAL					   8
#define RE_SHADER2D					   9
#define RE_SHADER3D					  10
#define RE_COFFEE_EXT				  11
#define RE_IO_IMAGE						12
#define RE_IO_MOVIE		        13
#define RE_IO_SCENE		        14
#define RE_IO_PREF	         	15
#define RE_IO_COFFEE	        16
#define RE_IO_CATALOG         17
#define RE_IO_SOUND           18
#define RE_IO_IMAGESAVER			19
#define RE_IO_FCV							20
#define RE_IO_LAYOUT          21
#define RE_BITMAP_FILTER			22
#define RE_VIDEOPOST					23
#define RE_SCENEHOOK					24
#define RE_PAINTER						50

class BaseBitmap;

struct IconData
{
	BaseBitmap	*bmp;
	LONG				x,y,w,h;
};

#define CPluginKey PluginKey
#define CPluginSequence PluginSequence
#define CPluginTrack PluginTrack
#define CPluginObject PluginObject
#define CPluginTag PluginTag

#endif
