// Connection example

#include "c4d_file.h"
#include "c4d_objectplugin.h"
#include "c4d_general.h"
#include "c4d_memory.h"
#include "c4d_thread.h"
#include "c4d_baseobject.h"
#include "c4d_basedocument.h"
#include "c4d_basebitmap.h"
#include "c4d_basedraw.h"
#include "c4d_basetag.h"
#include "c4d_resource.h"
#include "c4d_symbols.h"
#include "c4d_gui.h"

#include "messiahConstants.h"
#include "messiahManager.h"

#include "Diagnostic.h"


static Bool EditSettings(PluginObject *op)
{
  // Use double clicked on object. Display your GUI...
  mm->interfaceRequested();

	return TRUE;
}


static BaseObject* GetVirtualObjects(PluginObject* op, HierarchyHelp* hh)
{

	// set the current fps //////////////////////////////////////////////////
	// not sure that this is the best place for it, but i can't get a document
	// from the BaseObject passed to mm->processObject(), i assume it's because
	// they're not real objects
	// ron
	BaseDocument *doc = NULL;
	doc = op->GetDocument();
	if(doc)
	{
		LONG fps = doc->GetFps();
		mm->m_fps = fps;
	}////////////////////////////////////////////////////////////////////////

  // Check dependences
  Bool dirty = op->CheckCache(hh) || op->IsDirty(DIRTY_DATA);
  op->NewDependenceList();
  BaseObject* obj = op->down();
  while (obj)
  {
    op->AddDependence(hh, obj);
    obj = GetNextHierarchyObject(obj, op);
  }
  dirty = dirty || !op->CompareDependenceList();

  // Try to get the current time in the document
  BaseTime t;
  if (op->GetDocument())
  {
    t = op->GetDocument()->GetTime();
  }

  // Create the object hierarchy
  BaseObject* res = NULL;
  if (dirty)
  {
    // If something has changed, just rebuild everything for now

    op->NewDependenceList();

    res = AllocObject(Onull);
    res->SetName(op->GetName());

    // Exactly as above, first clone everything as-is (now passing NULL)
    BaseObject* obj = op->down();
    while (obj)
    {
      BaseObject* clone = op->GetHierarchyClone(hh, obj, HCLONE_ASIS, NULL);
      if (clone) clone->InsertUnderLast(res);
      obj = obj->next();
    }

    // Then clone the tagged objects as polygons, and replace them in res
    obj = res->down();
    while (obj)
    {
      PluginTag* tag = GetMessiahTag(obj);
      if (tag)
      {
        // Tagged object found, clone it as polygons
        BaseObject* clone = op->GetHierarchyClone(hh, obj, HCLONE_ASPOLY, NULL);

        // Add the tag if it was lost
        if (!GetMessiahTag(clone))
        {
          BaseTag* tag = AllocTag(Tplugin, TAG_PLUGIN_ID, 0);
          clone->InsertTag(tag);
        }

        // We could do something about spline objects here
        // (They are lost currently, if they're attached.)

        // Do the swapping manouvre
        if (clone)
        {
          clone->InsertAfter(obj);
          obj->Remove();
          FreeObject(obj);
        }

        obj = GetNextHierarchyObject(clone, op, TRUE);
      }
      else
      {
        obj = GetNextHierarchyObject(obj, op, TRUE);
      }
    }

    // Notify the connection manager of the current hierarchy
    mm->notifyCurrentHierarchy(res);
  }
  else
  {
    // Otherwise we can use the cache

    op->TouchDependenceList();
    res = op->GetCache(hh);
  }

  // Now we hopefully have a hierarchy to deform
  if (res)
  {
    BaseObject* obj = res->down();
    while (obj)
    {
      PluginTag* tag = GetMessiahTag(obj);
      if (tag)
      {
        // Deform every tagged object (pass the time as well)
        mm->processObject(obj, t);
        obj = GetNextHierarchyObject(obj, op, TRUE);
      }
      else
      {
        obj = GetNextHierarchyObject(obj, op);
      }
    }
  }

  // A manual loop of all dependences
  op->NewDependenceList();
  obj = op->down();
  while (obj)
  {
    op->AddDependence(hh, obj);
    obj = GetNextHierarchyObject(obj, op);
  }

  return res;
}


static Bool MenuPrepare(PluginObject* op, BaseDocument* doc)
{
  // Initalize the object before it's added to the scene
  op->SetName(GeLoadString(IDS_CONNECTION_DEFAULT_OBJECT_NAME));

  return TRUE;
}


static Bool Message(PluginObject* op, LONG msg, void* data)
{
  GePrint("Connection Message(): " + LongToString(msg) + " (" + LongToString((LONG)data) + ")");
  return TRUE;
}
  


static Bool ReadSettings(PluginObject *op, HyperFile *hf, LONG level)
{
  if (!hf) return FALSE;

  if (level >= 0)
  {
	mm->loadTags(*hf);
    mm->loadScene(*hf);
	BaseObject *obj = op->down();
	while(obj)
	{
		obj = GetNextHierarchyObject(obj, op, TRUE);
	}
  }

  GePrint("ReadSettings");
  return TRUE;
}

static Bool WriteSettings(PluginObject *op, HyperFile *hf)
{
	if (!hf) return FALSE;

	// Find the filename for the current document
	Filename path;
	BaseDocument* doc = op->GetDocument();
	if (doc)
	{
	path = doc->GetPath();
	}
	else
	{
	path = GeGetStartupPath();
	}
	
	mm->saveTags(*hf, op);
	mm->saveScene(*hf, path);

	GePrint("WriteSettings");


	return TRUE;
}

///////////////////////////////////////////////////////////
// Noitfy when scene is clearing so that messiah can be cleared

Bool CreateManager();
void FreeManager();

static Bool AllocSettings(PluginObject *op)
{
	char *dummy = new char;

	op->SetPluginData(dummy);

	return TRUE;
}

static Bool CloneSettings(PluginObject *src, PluginObject *dst)
{
	return TRUE;
}

static void FreeSettings(PluginObject *op)
{
	char *dummy = 0;
	
	dummy = static_cast<char *>(op->GetPluginData());

	delete dummy;

	nodeClearAllElements(2);

	// reinitialize the manager
	FreeManager();
	CreateManager();

	return;
}
//////////////////////////////////////////////////////////////

static BaseBitmap *icon = NULL;
static BaseBitmap *icon_small = NULL;

extern void DumpCWD();

#include <direct.h>

Bool RegisterMessiahConnection(void)
{
	char buff[1024];
	LONG bufflen;

	AppendDiagnostic("PLGDIR\n");
	DumpCWD();
	_chdir("res");
	AppendDiagnostic("RESDIR\n");
	DumpCWD();
	_chdir("strings_us");
	AppendDiagnostic("STRDIR\n");
	DumpCWD();
	_chdir("..\\..");

  // Get plugin name
	String name = GeLoadString(IDS_CONNECTION_TEXT);
	bufflen = name.GetCStringLen() + 1;
	name.GetCString(buff, bufflen);
	AppendDiagnostic("LSTR ");
	AppendDiagnostic(buff);
	AppendDiagnostic("\n");	

  // Load icons
	icon = AllocBaseBitmap();
	if( !icon ) AppendDiagnostic("ALLOCERR_ICO\n");
	if (!icon || icon->Init(GeGetPluginPath() + String("res") + String("messiah.tif")) != IMAGE_OK)
	{
		AppendDiagnostic("RESERR_ICO\n");
		return FALSE;
	}

	icon_small = AllocBaseBitmap();
	if( !icon_small ) AppendDiagnostic("ALLOCERR_ICOSM\n");
	if (!icon_small || icon_small->Init(GeGetPluginPath()+String("res")+String("messiah_small.tif"))!=IMAGE_OK)
	{
		AppendDiagnostic("RESERR_ICOSM\n");
		return FALSE;
	}

  // Create object plugin struct
	OBJECTPLUGIN cs;
	ClearMem(&cs,sizeof(cs));

  cs.disklevel       = 1;

	cs.add_menu			  = TRUE;
	cs.icon						= icon;
	cs.icon_small			= icon_small;
	cs.info						= OBJECT_GENERATOR | OBJECT_INPUT;

  cs.MenuPrepare    = MenuPrepare;
	cs.GetVirtualObjects = GetVirtualObjects;

  cs.ReadSettings   = ReadSettings;
  cs.WriteSettings  = WriteSettings;

	cs.EditSettings		= EditSettings;

	cs.AllocSettings	= AllocSettings;
	cs.CloneSettings	= CloneSettings;
	cs.FreeSettings		= FreeSettings;

  cs.Message        = Message;

	// Register plugin
  AppendDiagnostic("BREG\n");
  Bool ret = GeRegisterPlugin(C4DPL_OBJECT, CONNECTION_PLUGIN_ID, name, &cs, sizeof(cs));

  if(!ret)
  {
	  AppendDiagnostic("EREG -1\n");
	  return FALSE;
  }

  AppendDiagnostic("EREG 0\n");
  return TRUE;
}

void FreeMessiahConnection(void)
{
	FreeBaseBitmap(icon);
	FreeBaseBitmap(icon_small);
}
