#include "ge_math.h"
#include "c4d_memory.h"
#include "messiahManager.h"
#include "messiahConstants.h"

#include "ConfigData.h"

#ifdef _DEBUG
#include <stdio.h>
#endif

messiahManager* mm = NULL;

Bool CreateManager()
{
  if (mm) return FALSE;
  mm = gNew messiahManager;
  return mm != NULL;
}

void FreeManager()
{
  gDelete(mm);
}

Bool messiahManager::loadTags(HyperFile &hf)
{
	LONG numTags = 0;

	if( !hf.ReadLong(&numTags) ) return FALSE;

	// allocate mem for node list
	m_loadIDs.reserve(numTags);

	for(int i = 0; i < numTags; i++)
	{
		LONG id = 0;
		if( !hf.ReadLong(&id) ) return FALSE;
		m_loadIDs.push_back(id);
	}

	return TRUE;
}

Bool messiahManager::saveTags(HyperFile &hf, PluginObject *op)
{
	LONG numTags = 0;
	BaseObject *obj = op->down();
	while(obj)
	{
		PluginTag *tag = GetMessiahTag(obj);
		if(tag)
			numTags++;
		obj = GetNextHierarchyObject(obj, op, TRUE);
	}

	// write number of tags to file
	if( !hf.WriteLong(numTags) ) return FALSE;

	obj = op->down();
	while(obj)
	{
		PluginTag *tag = GetMessiahTag(obj);
		if(tag)
		{
			connectionData* data = getConnectionFromObject(obj);
			if(data)
			{
				LONG id = data->m_messiahNode->ID;
				if(!hf.WriteLong(id)) return FALSE;
			}
		}

		obj = GetNextHierarchyObject(obj, op, TRUE);
	}
	return TRUE;
}


Bool messiahManager::saveScene(HyperFile& hf, const Filename& path)
{
	

 // LONG messiah_scene_type;

  // Get the current path
  String pathString = path.GetString();
  LONG len = pathString.GetCStringLen() + 1;
  CHAR* C4D_scene_file_path = static_cast<CHAR*>(GeAlloc(len));
  pathString.GetCString(C4D_scene_file_path, len);

  // Get text buffer
	CHAR* buff = messiahSceneFileCreateBuff();
#ifdef _DEBUG
	FILE *fp = fopen("messiahdump.fxs", "w");
	if(fp)
		fputs(buff, fp);
	fclose(fp);
#endif
  GeFree(C4D_scene_file_path);
	if (!buff) return FALSE;

	// Store using the hyper file system
//	if (!hf.WriteLong(messiah_scene_type)) return FALSE; // Embedded or separate messiah scene file
  if (!hf.WriteString(buff)) return FALSE; // The data string

  // Clear the buffer
	messiahSceneFileClearBuff(buff);

  return TRUE;
}


Bool messiahManager::loadScene(HyperFile& hf)
{
	GePrint("Loading Begin");

  // Load the string and create a CHAR buffer
  String str;
  if (!hf.ReadString(&str)) return FALSE;
  LONG len = str.GetCStringLen() + 1;
  CHAR* buff = static_cast<CHAR*>(GeAlloc(len));
  str.GetCString(buff, len);

  // added: ron
  m_justLoaded = messiahSceneFileLoadBuff(buff);


  // Clear the buffer
  GeFree(buff);

  // Remove this once the objects are actually loaded
  m_justLoaded = TRUE;

  GePrint("Loading End");

  return TRUE;
}

void messiahManager::interfaceRequested()
{

	messiahInterface(); //always non-modal

}

void messiahManager::notifyCurrentHierarchy(BaseObject* parent)
{
  if (!parent) return;

  // Reset flags, so we can know what datas are used
  resetFlags();

  // Find the tagged objects
  BaseObject* obj = parent->down();
  while (obj)
  {
    PluginTag* tag = GetMessiahTag(obj);
    if (tag)
    {
      GePrint("Notified of: " + obj->GetName());

      connectionData* data = getConnectionFromObject(obj);
      if (!data)
      {
		  GePrint("Connecting: " + obj->GetName());
        // New object, insert its node
        data = gNew connectionData;
        data->m_messiahNode = nodeLogElement(NULL, (void*) GetStringHash(obj->GetName()), NULL, NULL);

        m_connections.push_back(data);

        // Flag
        data->m_processed = TRUE;

        // Update C4D node
        data->replaceC4DNode(obj);
        
        if (!m_justLoaded)
        {
          // Prepare a C-string (UTF8!)
          String name = obj->GetName();
          LONG len = name.GetCStringLen() + 1;
          CHAR* buf = static_cast<CHAR*>(GeAlloc(len));
          name.GetCString(buf, len);
          
          // Create the messiah object
          if (data->m_messiahNode)
          {
            data->m_messiahNode->obj = nodeObjectCreate(buf, 0, 0);
            hostObjectGeometryCopy(data->m_messiahNode, 1);
          }
          
          //GeFree(buf);        
		  GePrint("Done Connecting: " + obj->GetName());
        }
      }
      else
      {
        // Flag
        data->m_processed = TRUE;

        // Update C4D node
        data->replaceC4DNode(obj);

        // Currently always worst-case update
        //nodeObjectRebuild(data->m_messiahNode->obj, 0); // flag?
      }

      obj = GetNextHierarchyObject(obj, parent, TRUE);
    }
    else
    {
      obj = GetNextHierarchyObject(obj, parent);
    }
  }
  
	// load the scene now that the node log is built
	if( m_justLoaded )
	{
		// set load ID's
		obj = parent->down();
		int i = 0;

		while(obj)
		{
			PluginTag *tag = GetMessiahTag(obj);
			if(tag)
			{
				connectionData *data = getConnectionFromObject(obj);
				if(data && data->m_messiahNode)
					data->m_messiahNode->load_ID = m_loadIDs[i];
				i++; // next taged object
			}

			obj = GetNextHierarchyObject(obj, parent, TRUE);
		}

		// load the scene
		messiahSceneLoad(MESSIAH_SCENE_LOAD);
		m_justLoaded = FALSE;
	}

	/*
  // Remove any unflagged (OK, so the STL grew slightly out of hand... :-)
  connectionPtrs toRemove;
  std::remove_copy_if(m_connections.begin(), m_connections.end(), 
    std::back_inserter(toRemove), connectionData::hasBeenProcesed);
  connectionPtrs::iterator last = 
    std::remove_if(m_connections.begin(), m_connections.end(), 
                   connectionData::hasNotBeenProcesed);
  m_connections.erase(last, m_connections.end());

  for (int i = 0; i < toRemove.size(); ++i)
  {
    nodeClearElement(toRemove[i]->m_messiahNode, 0);
    gDelete(toRemove[i]);
  }
  */ // temporarily remove

}

void messiahManager::processObject(BaseObject *obj, BaseTime const &t)
{
  if (!obj) return;

	ConfigData *config = GetConfigData();
	if(!config) return;

  connectionData* data = getConnectionFromObject(obj);
  if (data && data->m_messiahNode && data->m_messiahNode->obj)
  {
    // Notify messiah to update the internal scene if necessary
    // (The time is passed as seconds.)
	messiahCheckTime(t.Get() * m_fps, 1); 
    
    //Modify the object's points 
    // (important that the order is that same as hostObjectGeometryCopy)
    BaseObject* child = obj; // The first time it's not a "child"
    LONG processedPoints = 0;
    while (child)
    {
      LONG pointsAdded = 0;
      
      // Only process point objects
      if (child->GetLayerType() & L_pointobject)
      {
        Matrix mgi = !child->GetMg();
        
        PointObject* pobj = static_cast<PointObject*>(child);
        LONG pointCount = pobj->GetPointCount();
        Vector* points = pobj->GetPoint();
        for (int i = 0; i < pointCount; ++i)
        {
          Vector localPoint;
		  double mpoint[3] = {0};

          nodeObjectGetPoint(data->m_messiahNode->obj, i + processedPoints, 
            &mpoint[0], &mpoint[1], &mpoint[2]);

		  localPoint.x = (float) mpoint[0] * (1/config->scaleFactor);
		  localPoint.y = (float) mpoint[1] * (1/config->scaleFactor);
		  localPoint.z = (float) mpoint[2] * (1/config->scaleFactor);

		  points[i] = mgi * localPoint;
        }
        pointsAdded += pointCount;
      }
      
      processedPoints += pointsAdded;
      child = GetNextHierarchyObject(child, obj);
    }

  }
}


/*
void messiahManager::processObject(BaseObject *obj, BaseTime const &t)
{
	if( !obj) return;

	connectionData *data = getConnectionFromObject( obj );
	if( data && data->m_messiahNode && data->m_messiahNode->obj )
	{
		// set current time
		messiahCheckTime(t.Get() * m_fps, 1); 

		BaseObject *child = obj;
		LONG ProcessedPoints = 0;

		while(child)
		{
			LONG PointsAdded = 0;

			if (child->GetLayerType() & L_pointobject)
			{
				Matrix mgi = !child->GetMg();

				PointObject *pobj = static_cast<PointObject *>(obj);
				LONG pointCount = pobj->GetPointCount();
				Vector *points = pobj->GetPoint();

				for( int i = 0; i < pointCount; i++)
				{
					Vector localPoint;
					double mpoint[3] = {0};

					nodeObjectGetPoint(data->m_messiahNode->obj, i+ProcessedPoints, 
						&mpoint[0], &mpoint[1], &mpoint[2]);

					localPoint.x = (float) mpoint[0];
					localPoint.y = (float) mpoint[1];
					localPoint.z = (float) mpoint[2];

					points[i] = mgi * localPoint;

				}
				
				PointsAdded  += pointCount;

			}// if polyobject

			ProcessedPoints += PointsAdded;
			child = GetNextHierarchyObject(child, obj);

		}// while child
	}// if connection data
}
*/
BaseObject* messiahManager::getGeometryFromElement(void* element)
{
  if (!element) return NULL;

  for (int i = 0; i < m_connections.size(); ++i)
  {
    if (m_connections[i]->m_messiahNode &&
        m_connections[i]->m_messiahNode->element == element)
    {
      return m_connections[i]->m_c4dNode;
    }
  }
  return NULL;
}