#define HOST_MODULE
#include "messiahDLL.h"
#include "messiahHOST_.h"

#include <direct.h>
#include <string.h>
#include <windows.h>
#include <stdio.h>

//#include "Diagnostic.h"
extern void AppendDiagnostic(char const *string);

//################################################################################
//host* functions/callbacks
//################################################################################

HINSTANCE messiahlib = NULL;

#define	HOST_SET(v)			if (!strcmp(#v,global_name)) return &##v
#define	MESSIAH_GET(v)		v = messiahGlobal(#v)

//---------------------------------------------------------------------------
//	Name:	hostGlobal
//	Purpose:Required global pointer send function.  Called from messiah.
//---------------------------------------------------------------------------
__EXPORTVAR(void*) hostGlobal(char *global_name)
{
	if (!global_name) return NULL;

	HOST_SET(hostObjectGeometryCopy);
	HOST_SET(hostObjectGetPoints);
	HOST_SET(hostObjectRename);

	return NULL;
}


//---------------------------------------------------------------------------
//	Name:	hostGlobalsGetAll
//	Purpose:Required global pointer retrieval function.  Called from host.
//---------------------------------------------------------------------------
__EXPORTVAR(void) hostGlobalsGetAll(void)
{
	 MESSIAH_GET(messiah_current_frame);

	 MESSIAH_GET(messiah_Activated);
	 MESSIAH_GET(messiahStart);
	 MESSIAH_GET(messiahInterface);
	 MESSIAH_GET(messiahShutdown);
	 MESSIAH_GET(messiahCheckTime);

	 MESSIAH_GET(messiahSceneFileCreateBuff);
	 MESSIAH_GET(messiahSceneFileLoadBuff);
	 MESSIAH_GET(messiahSceneLoad);
	 MESSIAH_GET(messiahSceneClear);
	 MESSIAH_GET(messiahSceneFileFilterBuff);
	 MESSIAH_GET(messiahSceneFileClearBuff);

	 MESSIAH_GET(num_nodes);
	 MESSIAH_GET(node_log);

	 MESSIAH_GET(nodeLogElement);
	 MESSIAH_GET(nodeGetElement);
	 MESSIAH_GET(nodeGetFirst);
	 MESSIAH_GET(nodeGetLast);
	 MESSIAH_GET(nodeSearchElement);
	 MESSIAH_GET(nodeClearElement);
	 MESSIAH_GET(nodeClearAllElements);

	 MESSIAH_GET(nodeObjectCreate);
	 MESSIAH_GET(nodeObjectRename);
	 MESSIAH_GET(nodeObjectCreatePoint);
	 MESSIAH_GET(nodeObjectGetPoint);
	 MESSIAH_GET(nodeObjectCreateSurface);
	 MESSIAH_GET(nodeObjectSurfaceSet);
	 MESSIAH_GET(nodeObjectCreatePoly);
	 MESSIAH_GET(nodeObjectAppend);
	 MESSIAH_GET(nodeObjectEnd);
	 MESSIAH_GET(nodeObjectRebuild);

	 MESSIAH_GET(nodeObjectGetMotion);
	 MESSIAH_GET(nodeObjectSetMotion);
	 MESSIAH_GET(nodeObjectGetWorldPos);
	 MESSIAH_GET(nodeObjectGetWorldRot);
}


//---------------------------------------------------------------------------
//	Name:	hostLoadmessiahHOST
//	Purpose:Required messiahHOST module loader.  Called from host.
//---------------------------------------------------------------------------

#include "ConfigData.h"

__EXPORTVAR(int) hostLoadmessiahHOST(void)
{

	char output[2048];
	char curDir[MAX_PATH];

	char libfile[] = "\\messiahHOST.dll";
	char libpath[256];

	ConfigData *data;

	data = GetConfigData();

	AppendDiagnostic("MPATH ");
	AppendDiagnostic(data->messiahPath);
	AppendDiagnostic("\n");

	strcpy(libpath, data->messiahPath);
	strcat(libpath, libfile);

	AppendDiagnostic("LIBPATH ");
	AppendDiagnostic(libpath);
	AppendDiagnostic("\n");

	AppendDiagnostic("BLLIB\n");
	if( messiahlib = LoadLibrary(libpath) )
	{
		AppendDiagnostic("ELLIB 0\n");
		AppendDiagnostic("BLGLOB\n");
		if( messiahGlobal = (void *) GetProcAddress(messiahlib, "messiahGlobal") )
		{
			AppendDiagnostic("ELGLOB 0\n");
			hostGlobalsGetAll();
		}
		else
		{
			AppendDiagnostic("ELGLOB -1\n");
			FreeLibrary(messiahlib);
			messiahlib = NULL;
			return FALSE;
		}
		return TRUE;
	}
	else
	{
		AppendDiagnostic("ELLIB -1\n");
		_getcwd(curDir, MAX_PATH);
		sprintf(output, "Unable to open messiahHOST.dll at location:\n%s\n%s", libpath, curDir);
		MessageBox(NULL, output, "messiah Error", 0);
	}

	return FALSE;
}

//---------------------------------------------------------------------------
//	Name:	hostUnLoadmessiahHOST
//	Purpose:Required messiahHOST module unloader.  Called from host.
//---------------------------------------------------------------------------
__EXPORTVAR(void) hostUnLoadmessiahHOST(void)
{
	if (messiahlib)
	{
		messiahShutdown();

		FreeLibrary(messiahlib);
		messiahlib = NULL;
	}
}


//---------------------------------------------------------------------------
//	Name:	hostObjectRename
//	Purpose:Required host object rename function
//---------------------------------------------------------------------------
__EXPORTVAR(int) hostObjectRename(FX_NodeLog *nd, int flags)
{

	return TRUE;
}


//---------------------------------------------------------------------------
//	Name:	hostObjectGeometryCopy
//	Purpose:Required host object geometry creation function
//---------------------------------------------------------------------------
__EXPORTVAR(int) hostObjectGeometryCopy(FX_NodeLog *nd, int add_to_hierarchy)
{
	return hostObjectGeometryCopy_(nd, add_to_hierarchy);
}


//---------------------------------------------------------------------------
//	Name:	hostObjectGetPoints
//	Purpose:Required host object point modification function
//---------------------------------------------------------------------------
__EXPORTVAR(int) hostObjectGetPoints(FX_NodeLog *nd, int reset_object)
{
	return TRUE;
}



